@@ -301,16 +301,12 @@ void CAICallback::ReleasedSharedMemArea(char* name)
 
 int CAICallback::CreateGroup()
 {
-	GML_RECMUTEX_LOCK(group); // CreateGroup
-
 	const CGroup* g = gh->CreateNewGroup();
 	return g->id;
 }
 
 void CAICallback::EraseGroup(int groupId)
 {
-	GML_RECMUTEX_LOCK(group); // EraseGroup
-
 	if (CHECK_GROUPID(groupId)) {
 		if (gh->groups[groupId]) {
 			gh->RemoveGroup(gh->groups[groupId]);
@@ -324,8 +320,6 @@ bool CAICallback::AddUnitToGroup(int unitId, int groupId)
 
 	CUnit* unit = GetMyTeamUnit(unitId);
 	if (unit) {
-		GML_RECMUTEX_LOCK(group); // AddUnitToGroup
-
 		if (CHECK_GROUPID(groupId) && gh->groups[groupId]) {
 			added = unit->SetGroup(gh->groups[groupId]);
 		}
@@ -1055,7 +1049,7 @@ const unsigned char* CAICallback::GetMetalMap()
 
 float CAICallback::GetElevation(float x, float z)
 {
-	return ground->GetHeightReal(x, z);
+	return CGround::GetHeightReal(x, z);
 }
 
 
@@ -1144,8 +1138,6 @@ void CAICallback::DrawUnit(const char* unitName, const float3& pos,
 	tdu.facing = facing;
 	std::pair<int, CUnitDrawer::TempDrawUnit> tp(gs->frameNum + lifetime, tdu);
 
-	GML_STDMUTEX_LOCK(temp); // DrawUnit
-
 	if (transparent) {
 		unitDrawer->tempTransparentDrawUnits.insert(tp);
 	} else {
@@ -1454,34 +1446,33 @@ int CAICallback::HandleCommand(int commandId, void* data)
 		} break;
 		case AIHCAddMapPointId: {
 			const AIHCAddMapPoint* cmdData = static_cast<AIHCAddMapPoint*>(data);
-			net->Send(CBaseNetProtocol::Get().SendMapDrawPoint(team, (short)cmdData->pos.x, (short)cmdData->pos.z, std::string(cmdData->label), false));
+			/*
+			   TODO: gu->myPlayerNum makes the command to look like as it comes from the local player,
+			   "team" should be used (but needs some major changes in other engine parts)
+			*/
+			net->Send(CBaseNetProtocol::Get().SendMapDrawPoint(gu->myPlayerNum, (short)cmdData->pos.x, (short)cmdData->pos.z, std::string(cmdData->label), false));
 			return 1;
 		} break;
 		case AIHCAddMapLineId: {
 			const AIHCAddMapLine* cmdData = static_cast<AIHCAddMapLine*>(data);
-			net->Send(CBaseNetProtocol::Get().SendMapDrawLine(team, (short)cmdData->posfrom.x, (short)cmdData->posfrom.z, (short)cmdData->posto.x, (short)cmdData->posto.z, false));
+			// see TODO above
+			net->Send(CBaseNetProtocol::Get().SendMapDrawLine(gu->myPlayerNum, (short)cmdData->posfrom.x, (short)cmdData->posfrom.z, (short)cmdData->posto.x, (short)cmdData->posto.z, false));
 			return 1;
 		} break;
 		case AIHCRemoveMapPointId: {
 			const AIHCRemoveMapPoint* cmdData = static_cast<AIHCRemoveMapPoint*>(data);
-			net->Send(CBaseNetProtocol::Get().SendMapErase(team, (short)cmdData->pos.x, (short)cmdData->pos.z));
+			// see TODO above
+			net->Send(CBaseNetProtocol::Get().SendMapErase(gu->myPlayerNum, (short)cmdData->pos.x, (short)cmdData->pos.z));
 			return 1;
 		} break;
-		case AIHCSendStartPosId: {
-			const AIHCSendStartPos* cmdData = static_cast<AIHCSendStartPos*>(data);
-			SendStartPos(cmdData->ready, cmdData->pos);
-			return 1;
-		} break;
-		case AIHCGetUnitDefByIdId: {
-			// NOTE: this command should never arrive, handled in SSkirmishAICallbackImpl
-			return 0;
-		} break;
-		case AIHCGetWeaponDefByIdId: {
-			// NOTE: this command should never arrive, handled in SSkirmishAICallbackImpl
-			return 0;
-		} break;
-		case AIHCGetFeatureDefByIdId: {
-			// NOTE: this command should never arrive, handled in SSkirmishAICallbackImpl
+		case AIHCSendStartPosId:
+		case AIHCGetUnitDefByIdId:
+		case AIHCGetWeaponDefByIdId:
+		case AIHCGetFeatureDefByIdId:
+		case AIHCGetDataDirId:
+		{
+			// NOTE: these commands should never arrive, handled in SSkirmishAICallbackImpl
+			assert(false);
 			return 0;
 		} break;
 
@@ -1547,15 +1538,6 @@ int CAICallback::HandleCommand(int commandId, void* data)
 			return 1;
 		} break;
 
-		case AIHCGetDataDirId: {
-			// do nothing
-			// this event will never end up here, as
-			// it is handled in the C layer directly
-			// see Clb_DataDirs_allocatePath in rts/ExternalAI/Interface/SSkirmishAICallback.h
-
-			return 0;
-		} break;
-
 		case AIHCDebugDrawId: {
 			AIHCDebugDraw* cmdData = static_cast<AIHCDebugDraw*>(data);
 
@@ -1771,7 +1753,6 @@ int CAICallback::GetSelectedUnits(int* unitIds, int unitIds_max)
 	verify();
 	int a = 0;
 
-	GML_RECMUTEX_LOCK(sel); // GetSelectedUnit
 	// check if the allyteam of the player running
 	// the AI lib matches the AI's actual allyteam
 	if (gu->myAllyTeam == teamHandler->AllyTeam(team)) {
