@@ -128,7 +128,6 @@ CCameraHandler::~CCameraHandler()
 
 void CCameraHandler::UpdateCam()
 {
-	GML_RECMUTEX_LOCK(cam); // UpdateCam
 
 	//??? a lot CameraControllers depend on the calling every frame the 1st part of the if-clause
 	//if (cameraTimeEnd < 0.0f)
@@ -160,8 +159,6 @@ void CCameraHandler::UpdateCam()
 
 void CCameraHandler::CameraTransition(float nsecs)
 {
-	GML_RECMUTEX_LOCK(cam); // CameraTransition
-
 	UpdateCam(); // prevents camera stutter when multithreading
 
 	nsecs = std::max(nsecs, 0.0f) * cameraTimeFactor;
@@ -178,7 +175,6 @@ void CCameraHandler::CameraTransition(float nsecs)
 
 void CCameraHandler::SetCameraMode(unsigned int mode)
 {
-	GML_RECMUTEX_LOCK(cam); // SetCameraMode
 
 	if ((mode >= camControllers.size()) || (mode == static_cast<unsigned int>(currCamCtrlNum))) {
 		return;
@@ -197,7 +193,6 @@ void CCameraHandler::SetCameraMode(unsigned int mode)
 
 void CCameraHandler::SetCameraMode(const std::string& modeName)
 {
-	GML_RECMUTEX_LOCK(cam); // SetCameraMode
 
 	const int modeNum = GetModeIndex(modeName);
 	if (modeNum >= 0) {
@@ -209,7 +204,6 @@ void CCameraHandler::SetCameraMode(const std::string& modeName)
 
 int CCameraHandler::GetModeIndex(const std::string& name) const
 {
-	GML_RECMUTEX_LOCK(cam); // GetModeIndex
 
 	std::map<std::string, unsigned int>::const_iterator it = nameMap.find(name);
 	if (it != nameMap.end()) {
@@ -221,7 +215,6 @@ int CCameraHandler::GetModeIndex(const std::string& name) const
 
 void CCameraHandler::PushMode()
 {
-	GML_RECMUTEX_LOCK(cam); // PushMode
 
 	controllerStack.push(GetCurrentControllerNum());
 }
@@ -229,7 +222,6 @@ void CCameraHandler::PushMode()
 
 void CCameraHandler::PopMode()
 {
-	GML_RECMUTEX_LOCK(cam); // PopMode
 
 	if (!controllerStack.empty()) {
 		SetCameraMode(controllerStack.top());
@@ -240,7 +232,6 @@ void CCameraHandler::PopMode()
 
 void CCameraHandler::ToggleState()
 {
-	GML_RECMUTEX_LOCK(cam); // ToggleState
 
 	CameraTransition(1.0f);
 
@@ -268,7 +259,6 @@ void CCameraHandler::ToggleState()
 
 void CCameraHandler::ToggleOverviewCamera()
 {
-	GML_RECMUTEX_LOCK(cam); // ToggleOverviewCamera
 
 	CameraTransition(1.0f);
 	if (controllerStack.empty()) {
@@ -283,7 +273,6 @@ void CCameraHandler::ToggleOverviewCamera()
 
 void CCameraHandler::SaveView(const std::string& name)
 {
-	GML_RECMUTEX_LOCK(cam); // SaveView
 
 	if (name.empty())
 		return;
@@ -297,7 +286,6 @@ void CCameraHandler::SaveView(const std::string& name)
 
 bool CCameraHandler::LoadView(const std::string& name)
 {
-	GML_RECMUTEX_LOCK(cam); // LoadView
 
 	if (name.empty()) {
 		return false;
@@ -329,7 +317,6 @@ bool CCameraHandler::LoadView(const std::string& name)
 
 void CCameraHandler::GetState(CCameraController::StateMap& sm) const
 {
-	GML_RECMUTEX_LOCK(cam); // GetState
 
 	sm.clear();
 	sm["mode"] = (float)currCamCtrlNum;
@@ -340,7 +327,6 @@ void CCameraHandler::GetState(CCameraController::StateMap& sm) const
 
 bool CCameraHandler::SetState(const CCameraController::StateMap& sm)
 {
-	GML_RECMUTEX_LOCK(cam); // SetState
 
 	CCameraController::StateMap::const_iterator it = sm.find("mode");
 	if (it != sm.end()) {
@@ -361,7 +347,6 @@ bool CCameraHandler::SetState(const CCameraController::StateMap& sm)
 
 const std::string CCameraHandler::GetCurrentControllerName() const
 {
-	GML_RECMUTEX_LOCK(cam); // GetCurrentControllerName
 
 	return currCamCtrl->GetName();
 }
@@ -369,7 +354,6 @@ const std::string CCameraHandler::GetCurrentControllerName() const
 
 void CCameraHandler::PushAction(const Action& action)
 {
-	GML_RECMUTEX_LOCK(cam); // PushAction
 
 	const std::string cmd = action.command;
 
@@ -408,6 +392,7 @@ void CCameraHandler::PushAction(const Action& action)
 			} else {
 				taCam->flipped = !taCam->flipped;
 			}
+			taCam->KeyMove(ZeroVector); //FIXME add a more clean way to force a call to ::UpdateVectors()
 		}
 		if (smCam) {
 			if (!action.extra.empty()) {
@@ -415,6 +400,7 @@ void CCameraHandler::PushAction(const Action& action)
 			} else {
 				smCam->flipped = !smCam->flipped;
 			}
+			smCam->KeyMove(ZeroVector); //FIXME add a more clean way to force a call to ::UpdateVectors()
 		}
 	}
 	else if (cmd == "viewsave") {
@@ -439,7 +425,6 @@ void CCameraHandler::PushAction(const Action& action)
 
 bool CCameraHandler::LoadViewData(const ViewData& vd)
 {
-	GML_RECMUTEX_LOCK(cam); // LoadViewData
 
 	if (vd.empty()) {
 		return false;
