@@ -4,14 +4,14 @@
 
 #include "System/Platform/byteorder.h"
 
-CR_BIND(PlayerStatistics, );
+CR_BIND(PlayerStatistics, )
 CR_REG_METADATA(PlayerStatistics, (
 	CR_MEMBER(mousePixels),
 	CR_MEMBER(mouseClicks),
 	CR_MEMBER(keyPresses),
 	CR_MEMBER(numCommands),
 	CR_MEMBER(unitCommands)
-));
+))
 
 
 PlayerStatistics::PlayerStatistics()
