@@ -42,7 +42,7 @@ public:
 			" commands to be usable") {}
 
 	bool Execute(const SyncedAction& action) const {
-		SetBoolArg(gs->cheatEnabled, action.GetArgs());
+		InverseOrSetBool(gs->cheatEnabled, action.GetArgs());
 		LogSystemStatus("Cheating", gs->cheatEnabled);
 		return true;
 	}
@@ -55,7 +55,7 @@ public:
 			"Enables/Disables widgets (LuaUI control)") {}
 
 	bool Execute(const SyncedAction& action) const {
-		SetBoolArg(gs->noHelperAIs, action.GetArgs());
+		InverseOrSetBool(gs->noHelperAIs, action.GetArgs());
 		selectedUnitsHandler.PossibleCommandChange(NULL);
 		LogSystemStatus("LuaUI control", gs->noHelperAIs);
 		return true;
@@ -70,7 +70,7 @@ public:
 
 	bool Execute(const SyncedAction& action) const {
 		bool disabled;
-		SetBoolArg(disabled, action.GetArgs());
+		InverseOrSetBool(disabled, action.GetArgs());
 		inMapDrawer->SetSpecMapDrawingAllowed(!disabled);
 		return true;
 	}
@@ -85,7 +85,7 @@ public:
 			"replays, which will DESYNC them)", true) {}
 
 	bool Execute(const SyncedAction& action) const {
-		SetBoolArg(gs->godMode, action.GetArgs());
+		InverseOrSetBool(gs->godMode, action.GetArgs());
 		CLuaUI::UpdateTeams();
 		LogSystemStatus("God-Mode", gs->godMode);
 		CPlayer::UpdateControlledTeams();
@@ -193,7 +193,7 @@ public:
 			"Enables/Disables spectators to use the chat") {}
 
 	bool Execute(const SyncedAction& action) const {
-		SetBoolArg(game->noSpectatorChat, action.GetArgs());
+		InverseOrSetBool(game->noSpectatorChat, action.GetArgs());
 		LogSystemStatus("Spectators chat", !game->noSpectatorChat);
 		return true;
 	}
@@ -232,7 +232,7 @@ public:
 
 	bool Execute(const SyncedAction& action) const {
 		bool devMode = CLuaHandle::GetDevMode();
-		SetBoolArg(devMode, action.GetArgs());
+		InverseOrSetBool(devMode, action.GetArgs());
 		CLuaHandle::SetDevMode(devMode);
 		LogSystemStatus("Lua dev-mode (can cause desyncs if enabled)", devMode);
 		return true;
@@ -247,7 +247,7 @@ public:
 			" through Lua", true) {}
 
 	bool Execute(const SyncedAction& action) const {
-		SetBoolArg(gs->editDefsEnabled, action.GetArgs());
+		InverseOrSetBool(gs->editDefsEnabled, action.GetArgs());
 		LogSystemStatus("Unit-, Feature- & Weapon-Def editing",
 				gs->editDefsEnabled);
 		return true;
@@ -283,14 +283,8 @@ public:
 				if (luaRules != NULL && arg == "enable") {
 					LOG_L(L_WARNING, "LuaRules is already loaded");
 				} else {
-					GML_MSTMUTEX_DOUNLOCK(sim); // temporarily unlock this mutex to prevent a deadlock
-					{
-						GML_STDMUTEX_LOCK(draw); // the draw thread accesses luaRules in too many places, so we lock the entire draw thread
-
-						CLuaRules::FreeHandler();
-						CLuaRules::LoadHandler();
-					}
-					GML_MSTMUTEX_DOLOCK(sim); // restore unlocked mutex
+					CLuaRules::FreeHandler();
+					CLuaRules::LoadHandler();
 
 					if (luaRules) {
 						LOG("LuaRules loaded");
@@ -303,18 +297,14 @@ public:
 			if (!gs->cheatEnabled) {
 				LOG_L(L_WARNING, "Cheating required to disable synced scripts");
 			} else {
-				GML_MSTMUTEX_DOUNLOCK(sim); // temporarily unlock this mutex to prevent a deadlock
-				{
-					GML_STDMUTEX_LOCK(draw); // the draw thread accesses luaRules in too many places, so we lock the entire draw thread
-
-					CLuaRules::FreeHandler();
-				}
-				GML_MSTMUTEX_DOLOCK(sim); // restore unlocked mutex
+				CLuaRules::FreeHandler();
 
 				LOG("LuaRules disabled");
 			}
 		} else if (luaRules) {
 			luaRules->GotChatMsg(arg, action.GetPlayerID());
+		} else {
+			LOG("LuaRules is not loaded");
 		}
 
 		return true;
@@ -368,7 +358,7 @@ public:
 		} else if (luaGaia) {
 			luaGaia->GotChatMsg(arg, action.GetPlayerID());
 		} else {
-			LOG("LuaGaia disabled");
+			LOG("LuaGaia is not loaded");
 		}
 
 		return true;
