@@ -49,7 +49,7 @@
 #include "System/TimeProfiler.h"
 #include "System/Input/KeyInput.h"
 #include "System/FileSystem/SimpleParser.h"
-#include "System/Sound/SoundChannels.h"
+#include "System/Sound/ISoundChannels.h"
 
 #include <boost/cstdint.hpp>
 
@@ -789,11 +789,9 @@ std::string CMiniMap::GetTooltip(int x, int y)
 		return buildTip;
 	}
 
-	{
-		const CUnit* unit = GetSelectUnit(GetMapPosition(x, y));
-		if (unit) {
-			return CTooltipConsole::MakeUnitString(unit);
-		}
+	const CUnit* unit = GetSelectUnit(GetMapPosition(x, y));
+	if (unit) {
+		return CTooltipConsole::MakeUnitString(unit);
 	}
 
 	const string selTip = selectedUnitsHandler.GetTooltip();
@@ -824,7 +822,7 @@ void CMiniMap::AddNotification(float3 pos, float3 color, float alpha)
 
 /******************************************************************************/
 
-void CMiniMap::DrawCircle(const float3& pos, float radius)
+void CMiniMap::DrawCircle(const float3& pos, float radius) const
 {
 	glPushMatrix();
 	glTranslatef(pos.x, pos.y, pos.z);
@@ -839,14 +837,13 @@ void CMiniMap::DrawCircle(const float3& pos, float radius)
 	glPopMatrix();
 }
 
-void CMiniMap::DrawSquare(const float3& pos, float xsize, float zsize)
+void CMiniMap::DrawSquare(const float3& pos, float xsize, float zsize) const
 {
 	float verts[] = {
 		pos.x + xsize, 0.0f, pos.z + zsize,
 		pos.x - xsize, 0.0f, pos.z + zsize,
 		pos.x - xsize, 0.0f, pos.z - zsize,
 		pos.x + xsize, 0.0f, pos.z - zsize
-
 	};
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glVertexPointer(3, GL_FLOAT, 0, verts);
@@ -859,12 +856,25 @@ void CMiniMap::DrawSurfaceCircle(const float3& pos, float radius, unsigned int)
 {
 	minimap->DrawCircle(pos, radius);
 }
+
+
 void CMiniMap::DrawSurfaceSquare(const float3& pos, float xsize, float ysize)
 {
 	minimap->DrawSquare(pos, xsize, ysize);
 }
 
 
+void CMiniMap::ApplyConstraintsMatrix() const
+{
+	if (!renderToTexture) {
+		glTranslatef(xpos * globalRendering->pixelX, ypos * globalRendering->pixelY, 0.0f);
+		glScalef(width * globalRendering->pixelX, height * globalRendering->pixelY, 1.0f);
+	}
+}
+
+
+/******************************************************************************/
+
 void CMiniMap::Update()
 {
 	if (minimized || width == 0 || height == 0)
@@ -881,91 +891,11 @@ void CMiniMap::Update()
 			nextDrawScreen = spring_gettime() + spring_msecs(1000.0f / refreshRate);
 
 			fbo.Bind();
-			if (minimapTexSize != int2(width, height)) {
-				minimapTexSize = int2(width, height);
-				multisampledFBO = (fbo.GetMaxSamples() > 1);
-
-				if (multisampledFBO) {
-					// multisampled FBO we are render to
-					fbo.Detach(GL_COLOR_ATTACHMENT0_EXT); // delete old RBO
-					fbo.CreateRenderBufferMultisample(GL_COLOR_ATTACHMENT0_EXT, GL_RGBA8, minimapTexSize.x, minimapTexSize.y, 4);
-					//fbo.CreateRenderBuffer(GL_DEPTH_ATTACHMENT_EXT, GL_DEPTH_COMPONENT16, minimapTexSize.x, minimapTexSize.y);
-
-					if (!fbo.CheckStatus("MINIMAP")) {
-						fbo.Detach(GL_COLOR_ATTACHMENT0_EXT);
-						multisampledFBO = false;
-					}
-				}
-
-				glDeleteTextures(1, &minimapTex);
-				glGenTextures(1, &minimapTex);
-
-				glBindTexture(GL_TEXTURE_2D, minimapTex);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, minimapTexSize.x, minimapTexSize.y, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
-
-				if (multisampledFBO) {
-					// resolve FBO with attached final texture target
-					fboResolve.Bind();
-					fboResolve.AttachTexture(minimapTex);
-
-					if (!fboResolve.CheckStatus("MINIMAP-RESOLVE")) {
-						renderToTexture = false;
-						return;
-					}
-				} else {
-					// directly render to texture without multisampling (fallback solution)
-					fbo.Bind();
-					fbo.AttachTexture(minimapTex);
-
-					if (!fbo.CheckStatus("MINIMAP-RESOLVE")) {
-						renderToTexture = false;
-						return;
-					}
-				}
-			}
+			if (minimapTexSize != int2(width, height))
+				ResizeTextureCache();
 
 			fbo.Bind();
-			// draws minimap into FBO
-			{
-				glMatrixMode(GL_PROJECTION);
-				glPushMatrix();
-				glLoadIdentity();
-				gluOrtho2D(0,1,0,1);
-				glMatrixMode(GL_MODELVIEW);
-				glPushMatrix();
-				glLoadIdentity();
-
-				const int2 oldPos(xpos, ypos);
-				xpos = 0.0f;
-				ypos = 0.0f;
-
-					glViewport(0, 0, minimapTexSize.x, minimapTexSize.y);
-					glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
-					glClear(GL_COLOR_BUFFER_BIT);
-					DrawForReal(false, true);
-
-				xpos = oldPos.x;
-				ypos = oldPos.y;
-
-				glMatrixMode(GL_PROJECTION);
-				glPopMatrix();
-				glMatrixMode(GL_MODELVIEW);
-				glPopMatrix();
-
-				// resolve multisampled FBO if there is one
-				if (multisampledFBO) {
-					glBindFramebufferEXT(GL_READ_FRAMEBUFFER, fbo.fboId);
-					glBindFramebufferEXT(GL_DRAW_FRAMEBUFFER, fboResolve.fboId);
-					glBlitFramebufferEXT(
-						0, 0, minimapTexSize.x, minimapTexSize.y,
-						0, 0, minimapTexSize.x, minimapTexSize.y,
-						GL_COLOR_BUFFER_BIT, GL_NEAREST);
-				}
-			}
+			UpdateTextureCache();
 
 			// no need, gets done in CGame
 			//fbo.Unbind();
@@ -974,48 +904,133 @@ void CMiniMap::Update()
 }
 
 
-void CMiniMap::Draw()
+void CMiniMap::ResizeTextureCache()
 {
-	if (!slaveDrawMode) {
-		{
-			glEnable(GL_BLEND);
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-			glPushAttrib(GL_DEPTH_BUFFER_BIT);
-			glDisable(GL_DEPTH_TEST);
-			glDepthFunc(GL_LEQUAL);
-			glDepthMask(GL_FALSE);
-			glDisable(GL_TEXTURE_2D);
-			glMatrixMode(GL_MODELVIEW);
+	minimapTexSize = int2(width, height);
+	multisampledFBO = (fbo.GetMaxSamples() > 1);
 
-			if (minimized) {
-				DrawMinimizedButton();
-				glPopAttrib();
-				glEnable(GL_TEXTURE_2D);
-				return;
-			}
+	if (multisampledFBO) {
+		// multisampled FBO we are render to
+		fbo.Detach(GL_COLOR_ATTACHMENT0_EXT); // delete old RBO
+		fbo.CreateRenderBufferMultisample(GL_COLOR_ATTACHMENT0_EXT, GL_RGBA8, minimapTexSize.x, minimapTexSize.y, 4);
+		//fbo.CreateRenderBuffer(GL_DEPTH_ATTACHMENT_EXT, GL_DEPTH_COMPONENT16, minimapTexSize.x, minimapTexSize.y);
 
-			// draw the frameborder
-			if (!globalRendering->dualScreenMode && !maximized) {
-				DrawFrame();
-			}
-
-			glPopAttrib();
+		if (!fbo.CheckStatus("MINIMAP")) {
+			fbo.Detach(GL_COLOR_ATTACHMENT0_EXT);
+			multisampledFBO = false;
 		}
+	}
 
-		DrawForReal(true);
+	glDeleteTextures(1, &minimapTex);
+	glGenTextures(1, &minimapTex);
+
+	glBindTexture(GL_TEXTURE_2D, minimapTex);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, minimapTexSize.x, minimapTexSize.y, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
+
+	if (multisampledFBO) {
+		// resolve FBO with attached final texture target
+		fboResolve.Bind();
+		fboResolve.AttachTexture(minimapTex);
+
+		if (!fboResolve.CheckStatus("MINIMAP-RESOLVE")) {
+			renderToTexture = false;
+			return;
+		}
+	} else {
+		// directly render to texture without multisampling (fallback solution)
+		fbo.Bind();
+		fbo.AttachTexture(minimapTex);
+
+		if (!fbo.CheckStatus("MINIMAP-RESOLVE")) {
+			renderToTexture = false;
+			return;
+		}
 	}
 }
 
 
-void CMiniMap::ApplyConstraintsMatrix() const
+void CMiniMap::UpdateTextureCache()
 {
-	if (!renderToTexture) {
-		glTranslatef(xpos * globalRendering->pixelX, ypos * globalRendering->pixelY, 0.0f);
-		glScalef(width * globalRendering->pixelX, height * globalRendering->pixelY, 1.0f);
+	// draws minimap into FBO
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glLoadIdentity();
+	gluOrtho2D(0,1,0,1);
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	glLoadIdentity();
+
+	const int2 oldPos(xpos, ypos);
+	xpos = 0.0f;
+	ypos = 0.0f;
+
+		glViewport(0, 0, minimapTexSize.x, minimapTexSize.y);
+		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
+		glClear(GL_COLOR_BUFFER_BIT);
+		DrawForReal(false, true);
+
+	xpos = oldPos.x;
+	ypos = oldPos.y;
+
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPopMatrix();
+
+	// resolve multisampled FBO if there is one
+	if (multisampledFBO) {
+		glBindFramebufferEXT(GL_READ_FRAMEBUFFER, fbo.fboId);
+		glBindFramebufferEXT(GL_DRAW_FRAMEBUFFER, fboResolve.fboId);
+		glBlitFramebufferEXT(
+			0, 0, minimapTexSize.x, minimapTexSize.y,
+			0, 0, minimapTexSize.x, minimapTexSize.y,
+			GL_COLOR_BUFFER_BIT, GL_NEAREST);
 	}
 }
 
 
+/******************************************************************************/
+
+void CMiniMap::Draw()
+{
+	if (slaveDrawMode)
+		return;
+
+	// Draw Border
+	{
+		glEnable(GL_BLEND);
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		glPushAttrib(GL_DEPTH_BUFFER_BIT);
+		glDisable(GL_DEPTH_TEST);
+		glDepthFunc(GL_LEQUAL);
+		glDepthMask(GL_FALSE);
+		glDisable(GL_TEXTURE_2D);
+		glMatrixMode(GL_MODELVIEW);
+
+		if (minimized) {
+			DrawMinimizedButton();
+			glPopAttrib();
+			glEnable(GL_TEXTURE_2D);
+			return;
+		}
+
+		// draw the frameborder
+		if (!globalRendering->dualScreenMode && !maximized) {
+			DrawFrame();
+		}
+
+		glPopAttrib();
+	}
+
+	// Draw Minimap itself
+	DrawForReal(true);
+}
+
+
 void CMiniMap::DrawForReal(bool use_geo, bool updateTex)
 {
 	SCOPED_TIMER("MiniMap::DrawForReal");
@@ -1025,40 +1040,8 @@ void CMiniMap::DrawForReal(bool use_geo, bool updateTex)
 
 	glActiveTexture(GL_TEXTURE0);
 
-	// Render `cached` minimap
-	if (renderToTexture && !updateTex) {
-		glPushAttrib(GL_COLOR_BUFFER_BIT);
-		glBindTexture(GL_TEXTURE_2D, minimapTex);
-		glEnable(GL_TEXTURE_2D);
-		glDisable(GL_BLEND);
-
-		if (use_geo) {
-			glPushMatrix();
-			glTranslatef(xpos * globalRendering->pixelX, ypos * globalRendering->pixelY, 0.0f);
-			glScalef(width * globalRendering->pixelX, height * globalRendering->pixelY, 1.0f);
-		}
-
-		glColor4f(1,1,1,1);
-		glBegin(GL_QUADS);
-			glTexCoord2f(0.0, 0.0); glVertex2f(0.0f, 0.0f);
-			glTexCoord2f(1.0, 0.0); glVertex2f(1.0f, 0.0f);
-			glTexCoord2f(1.0, 1.0); glVertex2f(1.0f, 1.0f);
-			glTexCoord2f(0.0, 1.0); glVertex2f(0.0f, 1.0f);
-		glEnd();
-
-		glEnable(GL_BLEND);
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-		DrawCameraFrustumAndMouseSelection();
-
-		if (use_geo)
-			glPopMatrix();
-
-		glDisable(GL_TEXTURE_2D);
-		glColor4f(1,1,1,1);
-		glPopAttrib();
+	if ((!updateTex) && RenderCachedTexture(use_geo))
 		return;
-	}
 
 	glPushAttrib(GL_DEPTH_BUFFER_BIT);
 	glEnable(GL_BLEND);
@@ -1086,179 +1069,43 @@ void CMiniMap::DrawForReal(bool use_geo, bool updateTex)
 	setSurfaceSquareFunc(DrawSurfaceSquare);
 	cursorIcons.Enable(false);
 
-	glColor4f(0.6f, 0.6f, 0.6f, 1.0f);
-
-	// don't mirror the map texture with flipped cameras
-	glMatrixMode(GL_TEXTURE);
-	glPushMatrix();
-	glLoadIdentity();
-	glMatrixMode(GL_MODELVIEW);
-
-	// draw the map
-	glDisable(GL_ALPHA_TEST);
-	glDisable(GL_BLEND);
-		readMap->DrawMinimap();
-	glEnable(GL_BLEND);
-
-	glMatrixMode(GL_TEXTURE);
-	glPopMatrix();
-	glMatrixMode(GL_MODELVIEW);
-
 	// clip everything outside of the minimap box
-	{
-		const double plane0[4] = {0,-1,0,1};
-		const double plane1[4] = {0,1,0,0};
-		const double plane2[4] = {-1,0,0,1};
-		const double plane3[4] = {1,0,0,0};
+	SetClipPlanes(false);
+	glEnable(GL_CLIP_PLANE0);
+	glEnable(GL_CLIP_PLANE1);
+	glEnable(GL_CLIP_PLANE2);
+	glEnable(GL_CLIP_PLANE3);
 
-		glClipPlane(GL_CLIP_PLANE0, plane0); // clip bottom
-		glClipPlane(GL_CLIP_PLANE1, plane1); // clip top
-		glClipPlane(GL_CLIP_PLANE2, plane2); // clip right
-		glClipPlane(GL_CLIP_PLANE3, plane3); // clip left
+	DrawBackground();
 
-		glEnable(GL_CLIP_PLANE0);
-		glEnable(GL_CLIP_PLANE1);
-		glEnable(GL_CLIP_PLANE2);
-		glEnable(GL_CLIP_PLANE3);
-	}
+	// allow the LUA scripts to overdraw the background image
+	SetClipPlanes(true);
+	eventHandler.DrawInMiniMapBackground();
+	SetClipPlanes(false);
 
-	// switch to top-down map/world coords (z is twisted with y compared to the real map/world coords)
-	glPushMatrix();
-	glTranslatef(0.0f, +1.0f, 0.0f);
-	glScalef(+1.0f / (gs->mapx * SQUARE_SIZE), -1.0f / (gs->mapy * SQUARE_SIZE), 1.0f);
+	DrawUnitIcons();
+	DrawWorldStuff();
 
-	glEnable(GL_TEXTURE_2D);
-	glEnable(GL_ALPHA_TEST);
-	glAlphaFunc(GL_GREATER, 0.0f);
-
-	{
-		unitDrawer->DrawUnitMiniMapIcons();
-	}
-
-	glDisable(GL_ALPHA_TEST);
-	glDisable(GL_TEXTURE_2D);
-
-	glPushMatrix();
-	glRotatef(-90.0f, +1.0f, 0.0f, 0.0f); // real 'world' coordinates
-	glScalef(1.0f, 0.0f, 1.0f); // skip the y-coord (Lua's DrawScreen is perspective and so any z-coord in it influence the x&y, too)
-
-	// draw the projectiles
-	if (drawProjectiles) {
-		glPointSize(1.0f);
-		WorkaroundATIPointSizeBug();
-		projectileDrawer->DrawProjectilesMiniMap();
-	}
-
-	// draw the queued commands
-	//
-	// NOTE: this needlessly adds to the CursorIcons list, but at least
-	//       they are not drawn  (because the input receivers are drawn
-	//       after the command queues)
-
-	LuaUnsyncedCtrl::DrawUnitCommandQueues();
-	if ((drawCommands > 0) && guihandler->GetQueueKeystate()) {
-		selectedUnitsHandler.DrawCommands();
-	}
-
-	glLineWidth(2.5f);
-	lineDrawer.DrawAll();
-	glLineWidth(1.0f);
-
-	// draw the selection shape, and some ranges
-	if (drawCommands > 0) {
-		guihandler->DrawMapStuff(!!drawCommands);
-	}
-
-	{
-		// draw unit ranges
-		const float radarSquare = radarHandler->radarDiv;
-		CUnitSet& selUnits = selectedUnitsHandler.selectedUnits;
-		for(CUnitSet::iterator si = selUnits.begin(); si != selUnits.end(); ++si) {
-			CUnit* unit = *si;
-			if (unit->radarRadius && !unit->beingBuilt && unit->activated) {
-				glColor3fv(cmdColors.rangeRadar);
-				DrawCircle(unit->pos, (unit->radarRadius * radarSquare));
-			}
-			if (unit->sonarRadius && !unit->beingBuilt && unit->activated) {
-				glColor3fv(cmdColors.rangeSonar);
-				DrawCircle(unit->pos, (unit->sonarRadius * radarSquare));
-			}
-			if (unit->jammerRadius && !unit->beingBuilt && unit->activated) {
-				glColor3fv(cmdColors.rangeJammer);
-				DrawCircle(unit->pos, (unit->jammerRadius * radarSquare));
-			}
-			// change if someone someday create a non stockpiled interceptor
-			const CWeapon* w = unit->stockpileWeapon;
-			if((w != NULL) && w->weaponDef->interceptor) {
-				if (w->numStockpiled) {
-					glColor3fv(cmdColors.rangeInterceptorOn);
-				} else {
-					glColor3fv(cmdColors.rangeInterceptorOff);
-				}
-				DrawCircle(unit->pos, w->weaponDef->coverageRange);
-			}
-		}
-	}
-
-	glPopMatrix(); // revert to the 2d xform
-
-	glMatrixMode(GL_MODELVIEW);
 	if (use_geo) {
 		glPopMatrix();
 	}
-	glPopMatrix();
 	glPopAttrib();
 	glEnable(GL_TEXTURE_2D);
 
-	{
-		// prepare ClipPlanes for Lua's DrawInMinimap Modelview matrix
-
-		// quote from glClipPlane spec:
-		// "When glClipPlane is called, equation is transformed by the inverse of the modelview matrix and stored in the resulting eye coordinates.
-		//  Subsequent changes to the modelview matrix have no effect on the stored plane-equation components."
-		// -> we have to use the same modelview matrix when calling glClipPlane and later draw calls
-
-		// set the modelview matrix to the same as used in Lua's DrawInMinimap
-		glPushMatrix();
-		glLoadIdentity();
-		glScalef(1.0f / width, 1.0f / height, 1.0f);
-
-		const double plane0[4] = {0, -1, 0, double(height)};
-		const double plane1[4] = {0, 1, 0, 0};
-		const double plane2[4] = {-1, 0, 0, double(width)};
-		const double plane3[4] = {1, 0, 0, 0};
-
-		glClipPlane(GL_CLIP_PLANE0, plane0); // clip bottom
-		glClipPlane(GL_CLIP_PLANE1, plane1); // clip top
-		glClipPlane(GL_CLIP_PLANE2, plane2); // clip right
-		glClipPlane(GL_CLIP_PLANE3, plane3); // clip left
-
-		glPopMatrix();
-	}
-
-	glMatrixMode(GL_TEXTURE); {
-		glPushMatrix();
-	}
-	glMatrixMode(GL_PROJECTION); {
-		glPushMatrix();
-	}
-	glMatrixMode(GL_MODELVIEW); {
-		glPushMatrix();
-	}
-
 	// allow the LUA scripts to draw into the minimap
+	SetClipPlanes(true);
 	eventHandler.DrawInMiniMap();
 
-	glMatrixMode(GL_TEXTURE); {
-		glPopMatrix();
-	}
-	glMatrixMode(GL_PROJECTION); {
-		glPopMatrix();
-	}
-	glMatrixMode(GL_MODELVIEW); {
+	if (!updateTex) {
+		glPushMatrix();
+			glTranslatef(xpos * globalRendering->pixelX, ypos * globalRendering->pixelY, 0.0f);
+			glScalef(width * globalRendering->pixelX, height * globalRendering->pixelY, 1.0f);
+			DrawCameraFrustumAndMouseSelection();
 		glPopMatrix();
 	}
 
+	// Finish
+	// Reset of GL state
 	if (use_geo && globalRendering->dualScreenMode)
 		glViewport(globalRendering->viewPosX,0,globalRendering->viewSizeX,globalRendering->viewSizeY);
 
@@ -1271,41 +1118,25 @@ void CMiniMap::DrawForReal(bool use_geo, bool updateTex)
 	cursorIcons.Enable(true);
 	setSurfaceCircleFunc(NULL);
 	setSurfaceSquareFunc(NULL);
-
-	if (!updateTex) {
-		glPushMatrix();
-			glTranslatef(xpos * globalRendering->pixelX, ypos * globalRendering->pixelY, 0.0f);
-			glScalef(width * globalRendering->pixelX, height * globalRendering->pixelY, 1.0f);
-			DrawCameraFrustumAndMouseSelection();
-		glPopMatrix();
-	}
 }
 
 
+/******************************************************************************/
+/******************************************************************************/
+
 void CMiniMap::DrawCameraFrustumAndMouseSelection()
 {
 	glDisable(GL_TEXTURE_2D);
-	glPushMatrix();
 
 	// clip everything outside of the minimap box
-	{
-		const double plane0[4] = {0,-1,0,1};
-		const double plane1[4] = {0,1,0,0};
-		const double plane2[4] = {-1,0,0,1};
-		const double plane3[4] = {1,0,0,0};
-
-		glClipPlane(GL_CLIP_PLANE0, plane0); // clip bottom
-		glClipPlane(GL_CLIP_PLANE1, plane1); // clip top
-		glClipPlane(GL_CLIP_PLANE2, plane2); // clip right
-		glClipPlane(GL_CLIP_PLANE3, plane3); // clip left
-
-		glEnable(GL_CLIP_PLANE0);
-		glEnable(GL_CLIP_PLANE1);
-		glEnable(GL_CLIP_PLANE2);
-		glEnable(GL_CLIP_PLANE3);
-	}
+	SetClipPlanes(false);
+	glEnable(GL_CLIP_PLANE0);
+	glEnable(GL_CLIP_PLANE1);
+	glEnable(GL_CLIP_PLANE2);
+	glEnable(GL_CLIP_PLANE3);
 
 	// switch to top-down map/world coords (z is twisted with y compared to the real map/world coords)
+	glPushMatrix();
 	glTranslatef(0.0f, +1.0f, 0.0f);
 	glScalef(+1.0f / (gs->mapx * SQUARE_SIZE), -1.0f / (gs->mapy * SQUARE_SIZE), 1.0f);
 
@@ -1315,16 +1146,15 @@ void CMiniMap::DrawCameraFrustumAndMouseSelection()
 		cam2->ClipFrustumLines(true, -10000.0f, 400096.0f);
 
 		const std::vector<CCamera::FrustumLine> negSides = cam2->GetNegFrustumSides();
-		std::vector<CCamera::FrustumLine>::const_iterator fli;
 
 		CVertexArray* va = GetVertexArray();
 		va->Initialize();
 		va->EnlargeArrays(negSides.size() * 2, 0, VA_SIZE_2D0);
 
-		for (fli = negSides.begin(); fli != negSides.end(); ++fli) {
-			if (fli->minz < fli->maxz) {
-				va->AddVertex2dQ0(fli->base + (fli->dir * fli->minz), fli->minz);
-				va->AddVertex2dQ0(fli->base + (fli->dir * fli->maxz), fli->maxz);
+		for (auto& fl: negSides) {
+			if (fl.minz < fl.maxz) {
+				va->AddVertexQ2d0((fl.dir * fl.minz) + fl.base, fl.minz);
+				va->AddVertexQ2d0((fl.dir * fl.maxz) + fl.base, fl.maxz);
 			}
 		}
 
@@ -1353,10 +1183,10 @@ void CMiniMap::DrawCameraFrustumAndMouseSelection()
 		CVertexArray* va = GetVertexArray();
 		va->Initialize();
 		va->EnlargeArrays(4, 0, VA_SIZE_2D0);
-			va->AddVertex2dQ0(oldPos.x, oldPos.z);
-			va->AddVertex2dQ0(newPos.x, oldPos.z);
-			va->AddVertex2dQ0(newPos.x, newPos.z);
-			va->AddVertex2dQ0(oldPos.x, newPos.z);
+			va->AddVertexQ2d0(oldPos.x, oldPos.z);
+			va->AddVertexQ2d0(newPos.x, oldPos.z);
+			va->AddVertexQ2d0(newPos.x, newPos.z);
+			va->AddVertexQ2d0(oldPos.x, newPos.z);
 		va->DrawArray2d0(GL_LINE_LOOP);
 
 		glLineWidth(1.0f);
@@ -1555,6 +1385,8 @@ void CMiniMap::DrawNotes()
 			ni = notes.erase(ni);
 			continue;
 		}
+
+		const SColor color(ni->color[0], ni->color[1], ni->color[2], ni->color[3]);
 		for (int a = 0; a < 3; ++a) {
 			const float modage = age + a * 0.1f;
 			const float rot = modage * 3;
@@ -1570,8 +1402,6 @@ void CMiniMap::DrawNotes()
 			}
 			const float sinSize = fastmath::sin(rot) * size;
 			const float cosSize = fastmath::cos(rot) * size;
-
-			const SColor color(ni->color[0], ni->color[1], ni->color[2], ni->color[3]);
 			va->AddVertexC(float3(ni->pos.x + sinSize, ni->pos.z + cosSize, 0.0f),color);
 			va->AddVertexC(float3(ni->pos.x + cosSize, ni->pos.z - sinSize, 0.0f),color);
 			va->AddVertexC(float3(ni->pos.x + cosSize, ni->pos.z - sinSize, 0.0f),color);
@@ -1587,4 +1417,208 @@ void CMiniMap::DrawNotes()
 }
 
 
+
+bool CMiniMap::RenderCachedTexture(bool use_geo)
+{
+	if (!renderToTexture) {
+		return false;
+	}
+
+	glPushAttrib(GL_COLOR_BUFFER_BIT);
+	glBindTexture(GL_TEXTURE_2D, minimapTex);
+	glEnable(GL_TEXTURE_2D);
+	glDisable(GL_BLEND);
+
+	if (use_geo) {
+		glPushMatrix();
+		glTranslatef(xpos * globalRendering->pixelX, ypos * globalRendering->pixelY, 0.0f);
+		glScalef(width * globalRendering->pixelX, height * globalRendering->pixelY, 1.0f);
+	}
+
+	glColor4f(1,1,1,1);
+	glBegin(GL_QUADS);
+		glTexCoord2f(0.0, 0.0); glVertex2f(0.0f, 0.0f);
+		glTexCoord2f(1.0, 0.0); glVertex2f(1.0f, 0.0f);
+		glTexCoord2f(1.0, 1.0); glVertex2f(1.0f, 1.0f);
+		glTexCoord2f(0.0, 1.0); glVertex2f(0.0f, 1.0f);
+	glEnd();
+
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+	DrawCameraFrustumAndMouseSelection();
+
+	if (use_geo)
+		glPopMatrix();
+
+	glDisable(GL_TEXTURE_2D);
+	glColor4f(1,1,1,1);
+	glPopAttrib();
+	return true;
+}
+
+
+void CMiniMap::DrawBackground() const
+{
+	glColor4f(0.6f, 0.6f, 0.6f, 1.0f);
+	//glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
+
+	// don't mirror the map texture with flipped cameras
+	glMatrixMode(GL_TEXTURE);
+	glPushMatrix();
+	glLoadIdentity();
+	glMatrixMode(GL_MODELVIEW);
+
+		// draw the map
+		glDisable(GL_ALPHA_TEST);
+		glDisable(GL_BLEND);
+			readMap->DrawMinimap();
+		glEnable(GL_BLEND);
+
+	glMatrixMode(GL_TEXTURE);
+	glPopMatrix();
+	glMatrixMode(GL_MODELVIEW);
+
+	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
+}
+
+
+void CMiniMap::DrawUnitIcons() const
+{
+	// switch to top-down map/world coords (z is twisted with y compared to the real map/world coords)
+	glPushMatrix();
+	glTranslatef(0.0f, +1.0f, 0.0f);
+	glScalef(+1.0f / (gs->mapx * SQUARE_SIZE), -1.0f / (gs->mapy * SQUARE_SIZE), 1.0f);
+
+	glEnable(GL_TEXTURE_2D);
+	glEnable(GL_ALPHA_TEST);
+	glAlphaFunc(GL_GREATER, 0.0f);
+
+	unitDrawer->DrawUnitMiniMapIcons();
+
+	glDisable(GL_ALPHA_TEST);
+	glDisable(GL_TEXTURE_2D);
+
+	glPopMatrix();
+}
+
+
+void CMiniMap::DrawUnitRanges() const
+{
+	// draw unit ranges
+	const float radarSquare = radarHandler->radarDiv;
+	CUnitSet& selUnits = selectedUnitsHandler.selectedUnits;
+	for(const CUnit* unit: selUnits) {
+		// LOS Ranges
+		if (unit->radarRadius && !unit->beingBuilt && unit->activated) {
+			glColor3fv(cmdColors.rangeRadar);
+			DrawCircle(unit->pos, (unit->radarRadius * radarSquare));
+		}
+		if (unit->sonarRadius && !unit->beingBuilt && unit->activated) {
+			glColor3fv(cmdColors.rangeSonar);
+			DrawCircle(unit->pos, (unit->sonarRadius * radarSquare));
+		}
+		if (unit->jammerRadius && !unit->beingBuilt && unit->activated) {
+			glColor3fv(cmdColors.rangeJammer);
+			DrawCircle(unit->pos, (unit->jammerRadius * radarSquare));
+		}
+
+		// Interceptor Ranges
+		for (const CWeapon* w: unit->weapons) {
+			auto& wd = *w->weaponDef;
+			if ((w->range > 300) && wd.interceptor) {
+				if (w->numStockpiled || !wd.stockpile) {
+					glColor3fv(cmdColors.rangeInterceptorOn);
+				} else {
+					glColor3fv(cmdColors.rangeInterceptorOff);
+				}
+				DrawCircle(unit->pos, wd.coverageRange);
+			}
+		}
+	}
+}
+
+
+void CMiniMap::DrawWorldStuff() const
+{
+	glPushMatrix();
+	glTranslatef(0.0f, +1.0f, 0.0f);
+	glScalef(+1.0f / (gs->mapx * SQUARE_SIZE), -1.0f / (gs->mapy * SQUARE_SIZE), 1.0f);
+	glRotatef(-90.0f, +1.0f, 0.0f, 0.0f); // real 'world' coordinates
+	glScalef(1.0f, 0.0f, 1.0f); // skip the y-coord (Lua's DrawScreen is perspective and so any z-coord in it influence the x&y, too)
+
+	// draw the projectiles
+	if (drawProjectiles) {
+		glPointSize(1.0f);
+		WorkaroundATIPointSizeBug();
+		projectileDrawer->DrawProjectilesMiniMap();
+	}
+
+	// draw the queued commands
+	//
+	// NOTE: this needlessly adds to the CursorIcons list, but at least
+	//       they are not drawn  (because the input receivers are drawn
+	//       after the command queues)
+	LuaUnsyncedCtrl::DrawUnitCommandQueues();
+	if ((drawCommands > 0) && guihandler->GetQueueKeystate()) {
+		selectedUnitsHandler.DrawCommands();
+	}
+
+	glLineWidth(2.5f);
+	lineDrawer.DrawAll();
+	glLineWidth(1.0f);
+
+	// draw the selection shape, and some ranges
+	if (drawCommands > 0) {
+		guihandler->DrawMapStuff(!!drawCommands);
+	}
+
+	DrawUnitRanges();
+
+	glPopMatrix();
+}
+
+
+void CMiniMap::SetClipPlanes(const bool lua) const
+{
+	if (lua) {
+		// prepare ClipPlanes for Lua's DrawInMinimap Modelview matrix
+
+		// quote from glClipPlane spec:
+		// "When glClipPlane is called, equation is transformed by the inverse of the modelview matrix and stored in the resulting eye coordinates.
+		//  Subsequent changes to the modelview matrix have no effect on the stored plane-equation components."
+		// -> we have to use the same modelview matrix when calling glClipPlane and later draw calls
+
+		// set the modelview matrix to the same as used in Lua's DrawInMinimap
+		glPushMatrix();
+		glLoadIdentity();
+		glScalef(1.0f / width, 1.0f / height, 1.0f);
+
+		const double plane0[4] = {0, -1, 0, double(height)};
+		const double plane1[4] = {0, 1, 0, 0};
+		const double plane2[4] = {-1, 0, 0, double(width)};
+		const double plane3[4] = {1, 0, 0, 0};
+
+		glClipPlane(GL_CLIP_PLANE0, plane0); // clip bottom
+		glClipPlane(GL_CLIP_PLANE1, plane1); // clip top
+		glClipPlane(GL_CLIP_PLANE2, plane2); // clip right
+		glClipPlane(GL_CLIP_PLANE3, plane3); // clip left
+
+		glPopMatrix();
+	} else {
+		// clip everything outside of the minimap box
+		const double plane0[4] = {0,-1,0,1};
+		const double plane1[4] = {0,1,0,0};
+		const double plane2[4] = {-1,0,0,1};
+		const double plane3[4] = {1,0,0,0};
+
+		glClipPlane(GL_CLIP_PLANE0, plane0); // clip bottom
+		glClipPlane(GL_CLIP_PLANE1, plane1); // clip top
+		glClipPlane(GL_CLIP_PLANE2, plane2); // clip right
+		glClipPlane(GL_CLIP_PLANE3, plane3); // clip left
+	}
+}
+
+
+
 /******************************************************************************/
