@@ -11,7 +11,7 @@
 #include "Map/BaseGroundDrawer.h"
 #include "Map/HeightMapTexture.h"
 #include "Map/ReadMap.h"
-#include "Rendering/glFont.h"
+#include "Rendering/Fonts/glFont.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/IconHandler.h"
 #include "Rendering/ShadowHandler.h"
@@ -577,11 +577,11 @@ int2 LuaMatTexture::GetSize() const
 		case LUATEX_GRASS:
 			return int2(1024, 1024);
 		case LUATEX_FONT:
-			return int2(font->GetTexWidth(), font->GetTexHeight());
+			return int2(font->GetTextureWidth(), font->GetTextureHeight());
 		case LUATEX_FONTSMALL:
-			return int2(smallFont->GetTexWidth(), smallFont->GetTexHeight());
+			return int2(smallFont->GetTextureWidth(), smallFont->GetTextureHeight());
 		case LUATEX_MINIMAP:
-			if (readMap != NULL) { 
+			if (readMap != NULL) {
 				return readMap->GetMiniMapTextureSize();
 			}
 		break;
@@ -591,7 +591,7 @@ int2 LuaMatTexture::GetSize() const
 		case LUATEX_INFOTEX_MTLMAP:
 		case LUATEX_INFOTEX_HGTMAP:
 		case LUATEX_INFOTEX_BLKMAP: {
-			if (readMap != NULL) { 
+			if (readMap != NULL) {
 				return (readMap->GetGroundDrawer()->GetInfoTexSize());
 			}
 		}
@@ -602,7 +602,7 @@ int2 LuaMatTexture::GetSize() const
 		case LUATEX_MAP_GBUFFER_EMITTEX:
 		case LUATEX_MAP_GBUFFER_MISCTEX:
 		case LUATEX_MAP_GBUFFER_ZVALTEX: {
-			if (readMap != NULL) { 
+			if (readMap != NULL) {
 				return (readMap->GetGroundDrawer()->GetGeometryBuffer()->GetWantedSize(readMap->GetGroundDrawer()->DrawDeferred()));
 			}
 		}
