@@ -1,20 +1,18 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "lib/gml/gmlcnf.h"
-
-
 #include "LuaUI.h"
 
 #include "LuaInclude.h"
 
 #include "LuaUnsyncedCtrl.h"
+#include "LuaArchive.h"
 #include "LuaCallInCheck.h"
 #include "LuaConstGL.h"
 #include "LuaConstCMD.h"
 #include "LuaConstCMDTYPE.h"
 #include "LuaConstGame.h"
 #include "LuaSyncedRead.h"
-#include "LuaUnsyncedCall.h"
+#include "LuaInterCall.h"
 #include "LuaUnsyncedRead.h"
 #include "LuaFeatureDefs.h"
 #include "LuaUnitDefs.h"
@@ -51,11 +49,11 @@
 #include "System/Util.h"
 #include "lib/luasocket/src/luasocket.h"
 
+#include <mutex>
+#include <boost/thread/mutex.hpp>
 #include <stdio.h>
 #include <set>
 #include <cctype>
-#include <SDL_keysym.h>
-#include <SDL_mouse.h>
 
 CONFIG(bool, LuaSocketEnabled)
 	.defaultValue(true)
@@ -88,42 +86,43 @@ static const char* GetVFSMode()
 /******************************************************************************/
 /******************************************************************************/
 
+static boost::mutex m_singleton;
+
 
 void CLuaUI::LoadHandler()
 {
-	if (luaUI) {
-		return;
+	{
+		std::lock_guard<boost::mutex> lk(m_singleton);
+		if (luaUI) return;
+
+		luaUI = new CLuaUI();
 	}
 
-	new CLuaUI();
-
 	if (!luaUI->IsValid()) {
-		delete luaUI;
+		FreeHandler();
 	}
 }
 
 
 void CLuaUI::FreeHandler()
 {
-	static bool inFree = false;
-	if (!inFree) {
-		inFree = true;
-		delete luaUI;
-		inFree = false;
-	}
+	std::lock_guard<boost::mutex> lk(m_singleton);
+	if (!luaUI) return;
+
+	auto* inst = luaUI;
+	luaUI = NULL;
+	inst->KillLua();
+	delete inst;
 }
 
 
 /******************************************************************************/
 
 CLuaUI::CLuaUI()
-: CLuaHandle("LuaUI", LUA_HANDLE_ORDER_UI, true)
+: CLuaHandle("LuaUI", LUA_HANDLE_ORDER_UI, true, false)
 {
-	GML::SetLuaUIState(L_Sim);
 	luaUI = this;
 
-	BEGIN_ITERATE_LUA_STATES();
-
 	if (!IsValid()) {
 		return;
 	}
@@ -187,20 +186,16 @@ CLuaUI::CLuaUI()
 
 	AddBasicCalls(L); // into Global
 
-	lua_pushstring(L, "Script");
-	lua_rawget(L, -2);
-	LuaPushNamedCFunc(L, "UpdateCallIn", CallOutUnsyncedUpdateCallIn);
-	lua_pop(L, 1);
-
 	// load the spring libraries
 	if (!LoadCFunctions(L)                                                 ||
 	    !AddEntriesToTable(L, "VFS",         LuaVFS::PushUnsynced)         ||
 	    !AddEntriesToTable(L, "VFS",       LuaZipFileReader::PushUnsynced) ||
 	    !AddEntriesToTable(L, "VFS",       LuaZipFileWriter::PushUnsynced) ||
+	    !AddEntriesToTable(L, "VFS",         LuaArchive::PushEntries)      ||
 	    !AddEntriesToTable(L, "UnitDefs",    LuaUnitDefs::PushEntries)     ||
 	    !AddEntriesToTable(L, "WeaponDefs",  LuaWeaponDefs::PushEntries)   ||
 	    !AddEntriesToTable(L, "FeatureDefs", LuaFeatureDefs::PushEntries)  ||
-	    !AddEntriesToTable(L, "Script",      LuaUnsyncedCall::PushEntries) ||
+	    !AddEntriesToTable(L, "Script",      LuaInterCall::PushEntriesUnsynced) ||
 	    !AddEntriesToTable(L, "Script",      LuaScream::PushEntries)       ||
 	    !AddEntriesToTable(L, "Spring",      LuaSyncedRead::PushEntries)   ||
 	    !AddEntriesToTable(L, "Spring",      LuaUnsyncedCtrl::PushEntries) ||
@@ -226,25 +221,17 @@ CLuaUI::CLuaUI()
 	eventHandler.AddClient(this);
 
 	// update extra call-ins
-	UnsyncedUpdateCallIn(L, "WorldTooltip");
-	UnsyncedUpdateCallIn(L, "MapDrawCmd");
-
-	UpdateUnsyncedXCalls(L);
+	UpdateCallIn(L, "WorldTooltip");
+	UpdateCallIn(L, "MapDrawCmd");
 
 	lua_settop(L, 0);
-
-	END_ITERATE_LUA_STATES();
 }
 
 
 CLuaUI::~CLuaUI()
 {
-	if (L_Sim != NULL || L_Draw != NULL) {
-		Shutdown();
-		KillLua();
-	}
 	luaUI = NULL;
-	GML::SetLuaUIState(NULL);
+	if (guihandler) guihandler->LoadConfig("ctrlpanel.txt");
 }
 
 void CLuaUI::InitLuaSocket(lua_State* L) {
@@ -255,7 +242,7 @@ void CLuaUI::InitLuaSocket(lua_State* L) {
 	LUA_OPEN_LIB(L,luaopen_socket_core);
 
 	if (f.LoadStringData(code)){
-		LoadCode(L, code.c_str(), filename.c_str());
+		LoadCode(L, code, filename);
 	} else {
 		LOG_L(L_ERROR, "Error loading %s", filename.c_str());
 	}
@@ -274,59 +261,38 @@ string CLuaUI::LoadFile(const string& filename) const
 }
 
 
-bool CLuaUI::HasCallIn(lua_State *L, const string& name)
+static bool IsDisallowedCallIn(const string& name)
 {
-	if (!IsValid()) {
-		return false;
-	}
-
-	// never allow these calls
-	if (name == "Explosion") {
-		return false;
-	}
-
-	if (name == "CollectGarbage")
-		return true;
-
-	lua_getglobal(L, name.c_str());
-	if (!lua_isfunction(L, -1)) {
-		lua_pop(L, 1);
-		return false;
-	}
-	lua_pop(L, 1);
-	return true;
+	return
+		   (name == "Explosion")
+		|| (name == "DrawUnit")
+		|| (name == "DrawFeature")
+		|| (name == "DrawShield")
+		|| (name == "DrawProjectile")
+	;
 }
 
 
-bool CLuaUI::UnsyncedUpdateCallIn(lua_State *L, const string& name)
+bool CLuaUI::HasCallIn(lua_State* L, const string& name)
 {
-	// never allow this call-in
-	if (name == "Explosion") {
+	// never allow these calls
+	if (IsDisallowedCallIn(name))
 		return false;
-	}
 
-	if (HasCallIn(L, name)) {
-		eventHandler.InsertEvent(this, name);
-	} else {
-		eventHandler.RemoveEvent(this, name);
-	}
-
-	UpdateUnsyncedXCalls(L);
-
-	return true;
+	return CLuaHandle::HasCallIn(L, name);
 }
 
 
 void CLuaUI::UpdateTeams()
 {
 	if (luaUI) {
-		luaUI->SetFullCtrl(gs->godMode, true);
+		luaUI->SetFullCtrl(gs->godMode);
 		luaUI->SetCtrlTeam(gs->godMode ? AllAccessTeam :
-		                  (gu->spectating ? NoAccessTeam : gu->myTeam), true);
-		luaUI->SetFullRead(gu->spectatingFullView, true);
-		luaUI->SetReadTeam(luaUI->GetFullRead() ? AllAccessTeam : gu->myTeam, true);
-		luaUI->SetReadAllyTeam(luaUI->GetFullRead() ? AllAccessTeam : gu->myAllyTeam, true);
-		luaUI->SetSelectTeam(gu->spectatingFullSelect ? AllAccessTeam : gu->myTeam, true);
+		                  (gu->spectating ? NoAccessTeam : gu->myTeam));
+		luaUI->SetFullRead(gu->spectatingFullView);
+		luaUI->SetReadTeam(luaUI->GetFullRead() ? AllAccessTeam : gu->myTeam);
+		luaUI->SetReadAllyTeam(luaUI->GetFullRead() ? AllAccessTeam : gu->myAllyTeam);
+		luaUI->SetSelectTeam(gu->spectatingFullSelect ? AllAccessTeam : gu->myTeam);
 	}
 }
 
@@ -344,10 +310,8 @@ bool CLuaUI::LoadCFunctions(lua_State* L)
 	lua_rawset(L, -3)
 
 	REGISTER_LUA_CFUNC(SetShockFrontFactors);
-	REGISTER_LUA_CFUNC(SendToUnsynced);
 
 	lua_setglobal(L, "Spring");
-
 	return true;
 }
 
@@ -355,37 +319,22 @@ bool CLuaUI::LoadCFunctions(lua_State* L)
 /******************************************************************************/
 /******************************************************************************/
 
-void CLuaUI::Shutdown()
-{
-	LUA_CALL_IN_CHECK(L);
-	luaL_checkstack(L, 2, __FUNCTION__);
-	static const LuaHashString cmdStr("Shutdown");
-	if (!cmdStr.GetGlobalFunc(L)) {
-		return;
-	}
-
-	// call the routine
-	RunCallIn(cmdStr, 0, 0);
-
-	return;
-}
-
-
-bool CLuaUI::ConfigCommand(const string& command)
+bool CLuaUI::ConfigCommand(const string& command) //FIXME rename to fit event name
 {
 	LUA_CALL_IN_CHECK(L, true);
 	luaL_checkstack(L, 2, __FUNCTION__);
 	static const LuaHashString cmdStr("ConfigureLayout");
 	if (!cmdStr.GetGlobalFunc(L)) {
-		return true; // the call is not defined
+		return false; // the call is not defined
 	}
 
-	lua_pushstring(L, command.c_str());
+	lua_pushsstring(L, command);
 
 	// call the routine
-	if (!RunCallIn(cmdStr, 1, 0)) {
+	if (!RunCallIn(L, cmdStr, 1, 0)) {
 		return false;
 	}
+
 	return true;
 }
 
@@ -406,25 +355,19 @@ void CLuaUI::ShockFront(const float3& pos, float power, float areaOfEffect, cons
 
 	float3 gap = (camera->GetPos() - pos);
 
-#if defined(USE_GML) && GML_ENABLE_SIM
-	// WTF?
-	const float shockFrontDistMod = (GML::Enabled() && distMod != NULL)? *distMod : this->shockFrontDistAdj;
-#else
 	const float shockFrontDistMod = this->shockFrontDistAdj;
-#endif
 	float dist = gap.Length() + shockFrontDistMod;
 
 	if ((power /= (dist * dist)) < shockFrontMinPower && distMod == NULL)
 		return;
 
-	LUA_UI_BATCH_PUSH(, UIShockFrontEvent(pos, power, areaOfEffect, shockFrontDistMod))
 	LUA_CALL_IN_CHECK(L);
 
 	luaL_checkstack(L, 6, __FUNCTION__);
 	static const LuaHashString cmdStr("ShockFront");
 
 	if (!cmdStr.GetGlobalFunc(L)) {
-		haveShockFront = false;
+		haveShockFront = false; //FIXME improve in GameHelper.cpp and pipe instead through eventHandler?
 		return; // the call is not defined
 	}
 
@@ -442,7 +385,7 @@ void CLuaUI::ShockFront(const float3& pos, float power, float areaOfEffect, cons
 	lua_pushnumber(L, dir.z);
 
 	// call the routine
-	if (!RunCallIn(cmdStr, 4, 0)) {
+	if (!RunCallIn(L, cmdStr, 4, 0)) {
 		return;
 	}
 
@@ -450,60 +393,8 @@ void CLuaUI::ShockFront(const float3& pos, float power, float areaOfEffect, cons
 }
 
 
-void CLuaUI::ExecuteUIEventBatch() {
-	if (!UseEventBatch())
-		return;
-
-	std::vector<UIEventBase> lleb;
-	{
-		GML_STDMUTEX_LOCK(llbatch); // ExecuteUIEventBatch
-
-		if (luaUIEventBatch.empty())
-			return;
-
-		luaUIEventBatch.swap(lleb);
-	}
-
-	GML_SELECT_LUA_STATE();
-	GML_DRCMUTEX_LOCK(lua); // ExecuteUIEventBatch
-
-	if (Threading::IsSimThread())
-		Threading::SetLuaBatchThread(false);
-
-	for (std::vector<UIEventBase>::iterator i = lleb.begin(); i != lleb.end(); ++i) {
-		const UIEventBase& e = *i;
-		switch (e.GetID()) {
-			case SHOCK_FRONT: {
-				LUA_EVENT_CAST(UIShockFrontEvent, e); ShockFront(ee.GetPos(), ee.GetPower(), ee.GetAreaOfEffect(), ee.GetDistMod());
-			} break;
-			default: {
-				LOG_L(L_ERROR, "%s: Invalid Event %d", __FUNCTION__, e.GetID());
-			} break;
-		}
-	}
-
-	if (Threading::IsSimThread())
-		Threading::SetLuaBatchThread(true);
-}
-
 /******************************************************************************/
 
-bool CLuaUI::HasLayoutButtons()
-{
-	SELECT_LUA_STATE();
-	GML_DRCMUTEX_LOCK(lua); // HasLayoutButtons
-
-	luaL_checkstack(L, 2, __FUNCTION__);
-
-	static const LuaHashString cmdStr("LayoutButtons");
-	if (!cmdStr.GetGlobalFunc(L)) {
-		return false; // the call is not defined
-	}
-	lua_pop(L, 1);
-	return true;
-}
-
-
 bool CLuaUI::LayoutButtons(int& xButtons, int& yButtons,
                            const vector<CommandDescription>& cmds,
                            vector<int>& removeCmds,
@@ -525,10 +416,6 @@ bool CLuaUI::LayoutButtons(int& xButtons, int& yButtons,
 	buttonList.clear();
 	menuName = "";
 
-	GML_THRMUTEX_LOCK(unit, GML_DRAW); // LayoutButtons
-	GML_THRMUTEX_LOCK(feat, GML_DRAW); // LayoutButtons
-//	GML_THRMUTEX_LOCK(proj, GML_DRAW); // LayoutButtons
-
 	LUA_CALL_IN_CHECK(L, false);
 	luaL_checkstack(L, 6, __FUNCTION__);
 	const int top = lua_gettop(L);
@@ -548,7 +435,7 @@ bool CLuaUI::LayoutButtons(int& xButtons, int& yButtons,
 	}
 
 	// call the function
-	if (!RunCallIn(cmdStr, 4, LUA_MULTRET)) {
+	if (!RunCallIn(L, cmdStr, 4, LUA_MULTRET)) {
 		lua_settop(L, top);
 		return false;
 	}
@@ -826,109 +713,6 @@ bool CLuaUI::GetLuaCmdDescList(lua_State* L, int index,
 }
 
 
-/******************************************************************************/
-/******************************************************************************/
-//
-// Custom Call-in
-//
-
-bool CLuaUI::HasUnsyncedXCall(lua_State* srcState, const string& funcName)
-{
-	SELECT_LUA_STATE();
-#if (LUA_MT_OPT & LUA_MUTEX)
-	if (GML::Enabled() && srcState != L && SingleState()) {
-		GML_STDMUTEX_LOCK(xcall); // HasUnsyncedXCall
-
-		return unsyncedXCalls.find(funcName) != unsyncedXCalls.end();
-	}
-#endif
-
-	lua_getglobal(L, funcName.c_str());
-	const bool haveFunc = lua_isfunction(L, -1);
-	lua_pop(L, 1);
-	return haveFunc;
-}
-
-
-int CLuaUI::UnsyncedXCall(lua_State* srcState, const string& funcName)
-{
-#if (LUA_MT_OPT & LUA_MUTEX)
-	if (GML::Enabled()) {
-		SELECT_UNSYNCED_LUA_STATE();
-		if (srcState != L) {
-			DelayDataDump ddmp;
-
-			LuaUtils::ShallowDataDump sdd;
-			sdd.type = LUA_TSTRING;
-			sdd.data.str = new std::string;
-			*sdd.data.str = funcName;
-
-			ddmp.data.push_back(sdd);
-
-			LuaUtils::Backup(ddmp.dump, srcState, lua_gettop(srcState));
-
-			lua_settop(srcState, 0);
-
-			GML_STDMUTEX_LOCK(scall); // UnsyncedXCall
-
-			delayedCallsFromSynced.push_back(DelayDataDump());
-
-			DelayDataDump &ddb = delayedCallsFromSynced.back();
-			ddb.data.swap(ddmp.data);
-			ddb.dump.swap(ddmp.dump);
-			ddb.xcall = true;
-
-			return 0;
-		}
-	}
-#endif
-
-	LUA_CALL_IN_CHECK(L, 0);
-	const LuaHashString funcHash(funcName);
-	if (!funcHash.GetGlobalFunc(L)) {
-		return 0;
-	}
-
-	const int top = lua_gettop(L) - 1; // do not count the function
-
-	const bool diffStates = (srcState != L);
-
-	int retCount;
-
-	if (!diffStates) {
-		lua_insert(L, 1); // move the function to the beginning
-		// call the function
-		if (!RunCallIn(funcHash, top, LUA_MULTRET)) {
-			return 0;
-		}
-		retCount = lua_gettop(L);
-	}
-	else {
-		const int srcCount = lua_gettop(srcState);
-
-		LuaUtils::CopyData(L, srcState, srcCount);
-		const bool origDrawingState = LuaOpenGL::IsDrawingEnabled(L);
-		LuaOpenGL::SetDrawingEnabled(L, LuaOpenGL::IsDrawingEnabled(srcState));
-
-		// call the function
-		if (!RunCallIn(funcHash, srcCount, LUA_MULTRET)) {
-			LuaOpenGL::SetDrawingEnabled(L, origDrawingState);
-			return 0;
-		}
-		LuaOpenGL::SetDrawingEnabled(L, origDrawingState);
-		retCount = lua_gettop(L) - top;
-
-		lua_settop(srcState, 0);
-		if (retCount > 0) {
-			LuaUtils::CopyData(srcState, L, retCount);
-		}
-		lua_settop(L, top);
-	}
-
-	return retCount;
-}
-
-
 /******************************************************************************/
 /******************************************************************************/
 //
@@ -953,34 +737,5 @@ int CLuaUI::SetShockFrontFactors(lua_State* L)
 	return 0;
 }
 
-int CLuaUI::UpdateUnsyncedXCalls(lua_State* L)
-{
-#if (LUA_MT_OPT & LUA_MUTEX)
-	if (!GML::Enabled() || !SingleState() || L != L_Sim)
-#endif
-		return 0;
-
-	GML_STDMUTEX_LOCK(xcall); // UpdateUnsyncedXCalls
-
-	unsyncedXCalls.clear();
-
-	for (lua_pushnil(L); lua_next(L, LUA_GLOBALSINDEX) != 0; lua_pop(L, 1)) {
-		const int ktype = lua_type(L, -2);
-		const int vtype = lua_type(L, -1);
-		if (ktype == LUA_TSTRING && vtype == LUA_TFUNCTION) {
-			size_t len = 0;
-			const char* data = lua_tolstring(L, -2, &len);
-			if (len > 0) {
-				std::string name;
-				name.resize(len);
-				memcpy(&name[0], data, len);
-				unsyncedXCalls.insert(name);
-			}
-		}
-	}
-
-	return 0;
-}
-
 /******************************************************************************/
 /******************************************************************************/
