@@ -13,6 +13,7 @@
 
 #include "LuaInclude.h"
 
+#include "LuaConfig.h"
 #include "LuaDefs.h"
 #include "LuaHandle.h"
 #include "LuaUtils.h"
@@ -27,6 +28,7 @@
 #include "Sim/Features/Feature.h"
 #include "Sim/Features/FeatureHandler.h"
 #include "Sim/Misc/CategoryHandler.h"
+#include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/Misc/Wind.h"
@@ -160,26 +162,26 @@ static int UnitDefIndex(lua_State* L)
 	const void* userData = lua_touserdata(L, lua_upvalueindex(1));
 	const UnitDef* ud = static_cast<const UnitDef*>(userData);
 	const DataElement& elem = it->second;
-	const char* p = ((const char*)ud) + elem.offset;
+	const void* p = ((const char*)ud) + elem.offset;
 	switch (elem.type) {
 		case READONLY_TYPE: {
 			lua_rawget(L, 1);
 			return 1;
 		}
 		case INT_TYPE: {
-			lua_pushnumber(L, *((int*)p));
+			lua_pushnumber(L, *reinterpret_cast<const int*>(p));
 			return 1;
 		}
 		case BOOL_TYPE: {
-			lua_pushboolean(L, *((bool*)p));
+			lua_pushboolean(L, *reinterpret_cast<const bool*>(p));
 			return 1;
 		}
 		case FLOAT_TYPE: {
-			lua_pushnumber(L, *((float*)p));
+			lua_pushnumber(L, *reinterpret_cast<const float*>(p));
 			return 1;
 		}
 		case STRING_TYPE: {
-			lua_pushsstring(L, *((string*)p));
+			lua_pushsstring(L, *reinterpret_cast<const std::string*>(p));
 			return 1;
 		}
 		case FUNCTION_TYPE: {
@@ -224,7 +226,7 @@ static int UnitDefNewIndex(lua_State* L)
 
 	// Definition editing
 	const DataElement& elem = it->second;
-	const char* p = ((const char*)ud) + elem.offset;
+	const void* p = ((const char*)ud) + elem.offset;
 
 	switch (elem.type) {
 		case FUNCTION_TYPE:
@@ -551,6 +553,38 @@ static int ModelTable(lua_State* L, const void* data) {
 	return 1;
 }
 
+
+static int ColVolTable(lua_State* L, const void* data) {
+	auto cv = static_cast<const CollisionVolume*>(data);
+	assert(cv != NULL);
+
+	lua_newtable(L);
+	switch (cv->GetVolumeType()) {
+		case CollisionVolume::COLVOL_TYPE_ELLIPSOID:
+			HSTR_PUSH_STRING(L, "type", "ellipsoid");
+			break;
+		case CollisionVolume::COLVOL_TYPE_CYLINDER:
+			HSTR_PUSH_STRING(L, "type", "cylinder");
+			break;
+		case CollisionVolume::COLVOL_TYPE_BOX:
+			HSTR_PUSH_STRING(L, "type", "box");
+			break;
+	}
+
+	LuaPushNamedNumber(L, "scaleX", cv->GetScales().x);
+	LuaPushNamedNumber(L, "scaleY", cv->GetScales().y);
+	LuaPushNamedNumber(L, "scaleZ", cv->GetScales().z);
+	LuaPushNamedNumber(L, "offsetX", cv->GetOffsets().x);
+	LuaPushNamedNumber(L, "offsetY", cv->GetOffsets().y);
+	LuaPushNamedNumber(L, "offsetZ", cv->GetOffsets().z);
+	LuaPushNamedNumber(L, "boundingRadius", cv->GetBoundingRadius());
+	LuaPushNamedBool(L, "defaultToSphere",    cv->DefaultToSphere());
+	LuaPushNamedBool(L, "defaultToFootPrint", cv->DefaultToFootPrint());
+	LuaPushNamedBool(L, "defaultToPieceTree", cv->DefaultToPieceTree());
+	return 1;
+}
+
+
 #define TYPE_FUNC(FuncName, LuaType)                           \
 	static int FuncName(lua_State* L, const void* data)        \
 	{                                                          \
@@ -568,32 +602,32 @@ static int ModelTable(lua_State* L, const void* data) {
 		return 1;                                              \
 	}
 
-TYPE_FUNC(IsTransportUnit, boolean);
-TYPE_FUNC(IsImmobileUnit, boolean);
-TYPE_FUNC(IsBuildingUnit, boolean);
-TYPE_FUNC(IsBuilderUnit, boolean);
-TYPE_FUNC(IsMobileBuilderUnit, boolean);
-TYPE_FUNC(IsStaticBuilderUnit, boolean);
-TYPE_FUNC(IsFactoryUnit, boolean);
-TYPE_FUNC(IsExtractorUnit, boolean);
-TYPE_FUNC(IsGroundUnit, boolean);
-TYPE_FUNC(IsAirUnit, boolean);
-TYPE_FUNC(IsStrafingAirUnit, boolean);
-TYPE_FUNC(IsHoveringAirUnit, boolean);
-TYPE_FUNC(IsFighterAirUnit, boolean);
-TYPE_FUNC(IsBomberAirUnit, boolean);
+TYPE_FUNC(IsTransportUnit, boolean)
+TYPE_FUNC(IsImmobileUnit, boolean)
+TYPE_FUNC(IsBuildingUnit, boolean)
+TYPE_FUNC(IsBuilderUnit, boolean)
+TYPE_FUNC(IsMobileBuilderUnit, boolean)
+TYPE_FUNC(IsStaticBuilderUnit, boolean)
+TYPE_FUNC(IsFactoryUnit, boolean)
+TYPE_FUNC(IsExtractorUnit, boolean)
+TYPE_FUNC(IsGroundUnit, boolean)
+TYPE_FUNC(IsAirUnit, boolean)
+TYPE_FUNC(IsStrafingAirUnit, boolean)
+TYPE_FUNC(IsHoveringAirUnit, boolean)
+TYPE_FUNC(IsFighterAirUnit, boolean)
+TYPE_FUNC(IsBomberAirUnit, boolean)
 
-TYPE_MODEL_FUNC(ModelHeight, height);
-TYPE_MODEL_FUNC(ModelRadius, radius);
-TYPE_MODEL_FUNC(ModelMinx,   mins.x);
-TYPE_MODEL_FUNC(ModelMidx,   relMidPos.x);
-TYPE_MODEL_FUNC(ModelMaxx,   maxs.x);
-TYPE_MODEL_FUNC(ModelMiny,   mins.y);
-TYPE_MODEL_FUNC(ModelMidy,   relMidPos.y);
-TYPE_MODEL_FUNC(ModelMaxy,   maxs.y);
-TYPE_MODEL_FUNC(ModelMinz,   mins.z);
-TYPE_MODEL_FUNC(ModelMidz,   relMidPos.z);
-TYPE_MODEL_FUNC(ModelMaxz,   maxs.z);
+TYPE_MODEL_FUNC(ModelHeight, height)
+TYPE_MODEL_FUNC(ModelRadius, radius)
+TYPE_MODEL_FUNC(ModelMinx,   mins.x)
+TYPE_MODEL_FUNC(ModelMidx,   relMidPos.x)
+TYPE_MODEL_FUNC(ModelMaxx,   maxs.x)
+TYPE_MODEL_FUNC(ModelMiny,   mins.y)
+TYPE_MODEL_FUNC(ModelMidy,   relMidPos.y)
+TYPE_MODEL_FUNC(ModelMaxy,   maxs.y)
+TYPE_MODEL_FUNC(ModelMinz,   mins.z)
+TYPE_MODEL_FUNC(ModelMidz,   relMidPos.z)
+TYPE_MODEL_FUNC(ModelMaxz,   maxs.z)
 
 
 
@@ -667,6 +701,7 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FUNCTION("shieldWeaponDef",    ud.shieldWeaponDef,    WeaponDefToID);
 	ADD_FUNCTION("stockpileWeaponDef", ud.stockpileWeaponDef, WeaponDefToID);
 	ADD_FUNCTION("iconType",           ud.iconType,           SafeIconType);
+	ADD_FUNCTION("collisionVolume",    ud.collisionVolume,    ColVolTable);
 
 	ADD_FUNCTION("isTransport", ud, IsTransportUnit);
 	ADD_FUNCTION("isImmobile", ud, IsImmobileUnit);
@@ -695,6 +730,8 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_FUNCTION("midz",    ud, ModelMidz);
 	ADD_FUNCTION("maxz",    ud, ModelMaxz);
 
+
+
 	ADD_INT("id", ud.id);
 	ADD_INT("cobID", ud.cobID);
 
