@@ -59,7 +59,7 @@
 #include "System/Net/PackPacket.h"
 #include "System/Util.h"
 #include "System/Sound/ISound.h"
-#include "System/Sound/SoundChannels.h"
+#include "System/Sound/ISoundChannels.h"
 #include "System/FileSystem/FileHandler.h"
 #include "System/FileSystem/DataDirLocater.h"
 #include "System/FileSystem/FileSystem.h"
@@ -68,12 +68,9 @@
 
 #include <boost/cstdint.hpp>
 #include "System/Platform/Misc.h"
-#include "LuaHelper.h"
 
-#if !defined(HEADLESS) && !defined(NO_SOUND)
-	#include "System/Sound/EFX.h"
-	#include "System/Sound/EFXPresets.h"
-#endif
+#include "System/Sound/OpenAL/EFX.h"
+#include "System/Sound/OpenAL/EFXPresets.h"
 
 #include <map>
 #include <set>
@@ -82,7 +79,7 @@
 
 #include <fstream>
 
-#include <SDL_keysym.h>
+#include <SDL_clipboard.h>
 #include <SDL_mouse.h>
 
 using std::min;
@@ -204,6 +201,8 @@ bool LuaUnsyncedCtrl::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(SetMouseCursor);
 	REGISTER_LUA_CFUNC(WarpMouse);
 
+	REGISTER_LUA_CFUNC(SetClipboard);
+
 	REGISTER_LUA_CFUNC(SetCameraOffset);
 
 	REGISTER_LUA_CFUNC(UpdateInfoTexture);
@@ -357,11 +356,6 @@ void LuaUnsyncedCtrl::DrawUnitCommandQueues()
 
 	glLineWidth(cmdColors.QueuedLineWidth());
 
-	GML_RECMUTEX_LOCK(unit); // DrawUnitCommandQueues
-	GML_RECMUTEX_LOCK(feat); // DrawUnitCommandQueues
-	GML_STDMUTEX_LOCK(cai); // DrawUnitCommandQueues
-	GML_STDMUTEX_LOCK(dque); // DrawUnitCommandQueues
-
 	std::set<int>::const_iterator ui;
 
 	for (ui = drawCmdQueueUnits.begin(); ui != drawCmdQueueUnits.end(); ++ui) {
@@ -381,8 +375,6 @@ void LuaUnsyncedCtrl::DrawUnitCommandQueues()
 
 void LuaUnsyncedCtrl::ClearUnitCommandQueues()
 {
-	GML_STDMUTEX_LOCK(dque); // ClearUnitCommandQueues
-
 	drawCmdQueueUnits.clear();
 }
 
@@ -527,7 +519,7 @@ int LuaUnsyncedCtrl::PlaySoundFile(lua_State* L)
 		}
 
 		//! last argument (with and without pos/speed arguments) is the optional `sfx channel`
-		AudioChannelImpl* channel = &Channels::General;
+		IAudioChannel** channel = &Channels::General;
 		if (args >= index) {
 			if (lua_isstring(L, index)) {
 				string channelStr = lua_tostring(L, index);
@@ -559,12 +551,12 @@ int LuaUnsyncedCtrl::PlaySoundFile(lua_State* L)
 
 		if (pos_given) {
 			if (speed_given) {
-				channel->PlaySample(soundID, pos, speed, volume);
+				channel[0]->PlaySample(soundID, pos, speed, volume);
 			} else {
-				channel->PlaySample(soundID, pos, volume);
+				channel[0]->PlaySample(soundID, pos, volume);
 			}
 		} else
-			channel->PlaySample(soundID, volume);
+			channel[0]->PlaySample(soundID, volume);
 
 		success = true;
 	}
@@ -584,7 +576,7 @@ int LuaUnsyncedCtrl::PlaySoundStream(lua_State* L)
 	const float volume = luaL_optnumber(L, 2, 1.0f);
 	bool enqueue = luaL_optboolean(L, 3, false);
 
-	Channels::BGMusic.StreamPlay(soundFile, volume, enqueue);
+	Channels::BGMusic->StreamPlay(soundFile, volume, enqueue);
 
 	// .ogg files don't have sound ID's generated
 	// for them (yet), so we always succeed here
@@ -598,17 +590,17 @@ int LuaUnsyncedCtrl::PlaySoundStream(lua_State* L)
 
 int LuaUnsyncedCtrl::StopSoundStream(lua_State*)
 {
-	Channels::BGMusic.StreamStop();
+	Channels::BGMusic->StreamStop();
 	return 0;
 }
 int LuaUnsyncedCtrl::PauseSoundStream(lua_State*)
 {
-	Channels::BGMusic.StreamPause();
+	Channels::BGMusic->StreamPause();
 	return 0;
 }
 int LuaUnsyncedCtrl::SetSoundStreamVolume(lua_State* L)
 {
-	Channels::BGMusic.SetVolume(luaL_checkfloat(L, 1));
+	Channels::BGMusic->SetVolume(luaL_checkfloat(L, 1));
 	return 0;
 }
 
@@ -764,8 +756,6 @@ int LuaUnsyncedCtrl::AddWorldUnit(lua_State* L)
 
 int LuaUnsyncedCtrl::DrawUnitCommands(lua_State* L)
 {
-	GML_STDMUTEX_LOCK(dque); // DrawUnitCommands
-
 	if (lua_istable(L, 1)) {
 		const bool isMap = luaL_optboolean(L, 2, false);
 		const int unitArg = isMap ? -2 : -1;
@@ -1513,8 +1503,6 @@ int LuaUnsyncedCtrl::SetUnitNoMinimap(lua_State* L)
 
 int LuaUnsyncedCtrl::SetUnitNoSelect(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(sel); // SetUnitNoSelect
-
 	if (CLuaHandle::GetHandleUserMode(L)) {
 		return 0;
 	}
@@ -1843,6 +1831,17 @@ int LuaUnsyncedCtrl::SetMouseCursor(lua_State* L)
 	return 0;
 }
 
+/******************************************************************************/
+
+int LuaUnsyncedCtrl::SetClipboard(lua_State* L)
+{
+	if (!CLuaHandle::CheckModUICtrl(L)) {
+		return 0;
+	}
+
+	SDL_SetClipboardText(luaL_checkstring(L, 1));
+	return 0;
+}
 
 /******************************************************************************/
 
@@ -2029,7 +2028,7 @@ int LuaUnsyncedCtrl::Restart(lua_State* L)
 
 	std::vector<std::string> processArgs;
 
-	// arguments given by Lua code, if any
+	// arguments to Spring binary given by Lua code, if any
 	if (!springArguments.empty()) {
 		processArgs.push_back(springArguments);
 	}
@@ -2037,27 +2036,24 @@ int LuaUnsyncedCtrl::Restart(lua_State* L)
 	if (!scriptContents.empty()) {
 		// create file 'script.txt' with contents given by Lua code
 		std::ofstream scriptFile(scriptFullName.c_str());
+
 		scriptFile.write(scriptContents.c_str(), scriptContents.size());
 		scriptFile.close();
 
 		processArgs.push_back(scriptFullName);
 	}
 
+	LOG("[Spring.%s] Spring \"%s\" should be restarting", __FUNCTION__, springFullName.c_str());
+
 #ifdef _WIN32
-		// else OpenAL crashes when using execvp
-		ISound::Shutdown();
+	// else OpenAL crashes when using execvp
+	ISound::Shutdown();
 #endif
 
-	const std::string execError = Platform::ExecuteProcess(springFullName, processArgs);
-
-	if (execError.empty()) {
-		LOG("[Spring.%s] the game should be restarting", __FUNCTION__);
-		lua_pushboolean(L, true);
-	} else {
-		LOG_L(L_ERROR, "[Spring.%s] error %s", __FUNCTION__, execError.c_str());
-		lua_pushboolean(L, false);
-	}
+	Platform::ExecuteProcess(springFullName, processArgs);
 
+	// not reached on success
+	lua_pushboolean(L, false);
 	return 1;
 }
 
@@ -2079,10 +2075,7 @@ int LuaUnsyncedCtrl::SetWMIcon(lua_State* L)
 int LuaUnsyncedCtrl::SetWMCaption(lua_State* L)
 {
 	const std::string title = luaL_checksstring(L, 1);
-	const std::string titleShort = luaL_optsstring(L, 2, title);
-
-	WindowManagerHelper::SetCaption(title, titleShort);
-
+	WindowManagerHelper::SetCaption(title);
 	return 0;
 }
 
@@ -2165,8 +2158,6 @@ int LuaUnsyncedCtrl::SetUnitDefImage(lua_State* L)
 
 int LuaUnsyncedCtrl::SetUnitGroup(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // SetUnitGroup
-
 	if (!CLuaHandle::CheckModUICtrl(L)) {
 		return 0;
 	}
@@ -2761,9 +2752,9 @@ int LuaUnsyncedCtrl::SendSkirmishAIMessage(lua_State* L) {
 
 	luaL_checkstack(L, 2, __FUNCTION__);
 	lua_pushboolean(L, eoh->SendLuaMessages(aiTeam, inData, outData));
-	lua_createtable(L, outData.size(), 0);
 
 	// push the AI response(s)
+	lua_createtable(L, outData.size(), 0);
 	for (unsigned int n = 0; n < outData.size(); n++) {
 		lua_pushstring(L, outData[n]);
 		lua_rawseti(L, -2, n + 1);
