@@ -15,40 +15,13 @@
 
 #define CLAMP(i) Clamp((i), 0, smfReadMap->maxHeightMapIdx)
 
-#ifdef USE_GML
-#include "lib/gml/gmlsrv.h"
-extern gmlClientServer<void, int, CUnit*>* gmlProcessor;
-
-void CLegacyMeshDrawer::DoDrawGroundRowMT(void* c, int bty) {
-	((CLegacyMeshDrawer*) c)->DoDrawGroundRow(cam2, bty);
-}
-
-void CLegacyMeshDrawer::DoDrawGroundShadowLODMT(void* c, int nlod) {
-	((CLegacyMeshDrawer*) c)->DoDrawGroundShadowLOD(nlod);
-}
-
-CONFIG(bool, MultiThreadDrawGround).defaultValue(true);
-CONFIG(bool, MultiThreadDrawGroundShadow).defaultValue(false);
-
-#endif
-
-
 CLegacyMeshDrawer::CLegacyMeshDrawer(CSMFReadMap* rm, CSMFGroundDrawer* gd)
 	: smfReadMap(rm)
 	, smfGroundDrawer(gd)
 	, viewRadius(4)
 	, neededLod(4)
 	//, waterDrawn(false)
-#ifdef USE_GML
-	// handled by reference!
-	, multiThreadDrawGround(gd->multiThreadDrawGround)
-	, multiThreadDrawGroundShadow(gd->multiThreadDrawGroundShadow)
-#endif
 {
-#ifdef USE_GML
-	multiThreadDrawGround = configHandler->GetBool("MultiThreadDrawGround");
-	multiThreadDrawGroundShadow = configHandler->GetBool("MultiThreadDrawGroundShadow");
-#endif
 }
 
 CLegacyMeshDrawer::~CLegacyMeshDrawer()
@@ -76,10 +49,6 @@ void CLegacyMeshDrawer::DrawVertexAQ(CVertexArray* ma, int x, int y, float heigh
 	ma->AddVertexQ0(x * SQUARE_SIZE, height, y * SQUARE_SIZE);
 }
 
-void CLegacyMeshDrawer::EndStripQ(CVertexArray* ma)
-{
-	ma->EndStripQ();
-}
 
 void CLegacyMeshDrawer::DrawGroundVertexArrayQ(CVertexArray*& ma)
 {
@@ -265,7 +234,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 				int yhlod = y + hlod;
 				int nloop = (xe - xs) / lod + 1;
 
-				ma->EnlargeArrays((52 * nloop), 14 * nloop + 1);
+				ma->EnlargeArrays(52 * nloop);
 
 				int yhdx = y * smfReadMap->heightMapSizeX;
 				int ylhdx = yhdx + lod * smfReadMap->heightMapSizeX;
@@ -300,7 +269,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							float h4 = (GetVisibleVertexHeight(idx2) + GetVisibleVertexHeight(idx2LOD)) * hmocxp + GetVisibleVertexHeight(idx2HLOD) * oldcamxpart;
 
 							if (inStrip) {
-								EndStripQ(ma);
+								ma->EndStrip();
 								inStrip = false;
 							}
 
@@ -308,18 +277,18 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							DrawVertexAQ(ma, x, yhlod, h1);
 							DrawVertexAQ(ma, xhlod, y, h2);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
-							EndStripQ(ma);
+							ma->EndStrip();
 							DrawVertexAQ(ma, x, yhlod, h1);
 							DrawVertexAQ(ma, x, ylod);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, xhlod, ylod, h4);
-							EndStripQ(ma);
+							ma->EndStrip();
 							DrawVertexAQ(ma, xhlod, ylod, h4);
 							DrawVertexAQ(ma, xlod, ylod);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, xlod, y);
 							DrawVertexAQ(ma, xhlod, y, h2);
-							EndStripQ(ma);
+							ma->EndStrip();
 						}
 						else if ((x <= cx - vrhlod)) {
 							//! lower LOD to the left
@@ -332,7 +301,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							float h4 = (GetVisibleVertexHeight(idx2   ) + GetVisibleVertexHeight(idx2LOD)) * hocxp + GetVisibleVertexHeight(idx2HLOD) * mocxp;
 
 							if (inStrip) {
-								EndStripQ(ma);
+								ma->EndStrip();
 								inStrip = false;
 							}
 
@@ -340,18 +309,18 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							DrawVertexAQ(ma, xlod, y);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, xhlod, y, h2);
-							EndStripQ(ma);
+							ma->EndStrip();
 							DrawVertexAQ(ma, xlod, ylod);
 							DrawVertexAQ(ma, xlod, yhlod, h1);
 							DrawVertexAQ(ma, xhlod, ylod, h4);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
-							EndStripQ(ma);
+							ma->EndStrip();
 							DrawVertexAQ(ma, xhlod, y, h2);
 							DrawVertexAQ(ma, x, y);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, x, ylod);
 							DrawVertexAQ(ma, xhlod, ylod, h4);
-							EndStripQ(ma);
+							ma->EndStrip();
 						}
 
 						if ((y >= cy + vrhlod)) {
@@ -365,7 +334,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							float h4 = (GetVisibleVertexHeight(idx2LOD) + GetVisibleVertexHeight(idx1LOD)) * hmocyp + GetVisibleVertexHeight(idx3LOD ) * oldcamypart;
 
 							if (inStrip) {
-								EndStripQ(ma);
+								ma->EndStrip();
 								inStrip = false;
 							}
 
@@ -375,13 +344,13 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, xlod, y);
 							DrawVertexAQ(ma, xlod, yhlod, h4);
-							EndStripQ(ma);
+							ma->EndStrip();
 							DrawVertexAQ(ma, x, yhlod, h2);
 							DrawVertexAQ(ma, x, ylod);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, xlod, ylod);
 							DrawVertexAQ(ma, xlod, yhlod, h4);
-							EndStripQ(ma);
+							ma->EndStrip();
 						}
 						else if ((y <= cy - vrhlod)) {
 							//! lower LOD beneath
@@ -394,7 +363,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							float h4 = (GetVisibleVertexHeight(idx2LOD) + GetVisibleVertexHeight(idx1LOD)) * hocyp + GetVisibleVertexHeight(idx3LOD ) * mocyp;
 
 							if (inStrip) {
-								EndStripQ(ma);
+								ma->EndStrip();
 								inStrip = false;
 							}
 
@@ -404,19 +373,19 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							DrawVertexAQ(ma, xhlod, ylod, h1);
 							DrawVertexAQ(ma, xlod, yhlod, h4);
 							DrawVertexAQ(ma, xlod, ylod);
-							EndStripQ(ma);
+							ma->EndStrip();
 							DrawVertexAQ(ma, xlod, yhlod, h4);
 							DrawVertexAQ(ma, xlod, y);
 							DrawVertexAQ(ma, xhlod, yhlod, h3);
 							DrawVertexAQ(ma, x, y);
 							DrawVertexAQ(ma, x, yhlod, h2);
-							EndStripQ(ma);
+							ma->EndStrip();
 						}
 					}
 				}
 
 				if (inStrip) {
-					EndStripQ(ma);
+					ma->EndStrip();
 					inStrip = false;
 				}
 			} //for (y = ystart; y < yend; y += lod)
@@ -426,7 +395,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 			int nloop = (yed - yst) / lod + 1;
 
 			if (nloop > 0)
-				ma->EnlargeArrays((8 * nloop), 2 * nloop);
+				ma->EnlargeArrays(8 * nloop);
 
 			//! rita yttre begr?snings yta mot n?ta lod
 			if (maxlx < maxtx && maxlx >= mintx) {
@@ -451,7 +420,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 						DrawVertexAQ(ma, xlod, y);
 						DrawVertexAQ(ma, xlod, y + lod, h);
 					}
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 			}
 
@@ -476,7 +445,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 					}
 					DrawVertexAQ(ma, xlod, y);
 					DrawVertexAQ(ma, xlod, y + lod);
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 			}
 
@@ -492,7 +461,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 					int nloop = (xe - xs) / lod + 2; //! one extra for if statment
 					int ylhdx = (y + lod) * smfReadMap->heightMapSizeX;
 
-					ma->EnlargeArrays((2 * nloop), 1);
+					ma->EnlargeArrays(2 * nloop);
 
 					if (x % dlod) {
 						int idx2 = CLAMP(ylhdx + x), idx2PLOD = CLAMP(idx2 + lod), idx2MLOD = CLAMP(idx2 - lod);
@@ -514,7 +483,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							DrawVertexAQ(ma, x + lod, ylod, h);
 						}
 					}
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 			}
 
@@ -530,7 +499,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 					int yhdx = y * smfReadMap->heightMapSizeX;
 					int nloop = (xe - xs) / lod + 2; //! one extra for if statment
 
-					ma->EnlargeArrays((2 * nloop), 1);
+					ma->EnlargeArrays(2 * nloop);
 
 					if (x % dlod) {
 						int idx1 = CLAMP(yhdx + x), idx1PLOD = CLAMP(idx1 + lod), idx1MLOD = CLAMP(idx1 - lod);
@@ -553,7 +522,7 @@ void CLegacyMeshDrawer::DoDrawGroundRow(const CCamera* cam, int bty)
 							DrawVertexAQ(ma, x + lod, ylod);
 						}
 					}
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 			}
 
@@ -597,26 +566,6 @@ void CLegacyMeshDrawer::DrawMesh(const DrawPass::e& drawPass)
 	//waterDrawn = (drawPass == DrawPass::WaterReflection);
 
 	{ // profiler scope
-#ifdef USE_GML
-		// Profiler results, 4 threads: multiThreadDrawGround is faster only if ViewRadius is below 60 (probably depends on memory/cache speeds)
-		const bool mt = GML_PROFILER(multiThreadDrawGround)
-
-		if (mt) {
-			gmlProcessor->Work(
-				NULL,                                 // wrk
-				&CLegacyMeshDrawer::DoDrawGroundRowMT, // wrka
-				NULL,                                 // wrkit
-				this,                                 // cls
-				gmlThreadCount,                       // mt
-				FALSE,                                // sm
-				NULL,                                 // it
-				smfReadMap->numBigTexY,               // nu
-				50,                                   // l1
-				100,                                  // l2
-				TRUE                                  // sw
-			);
-		} else
-#endif
 		{
 			int camBty = math::floor(cam2->GetPos().z / (smfReadMap->bigSquareSize * SQUARE_SIZE));
 			camBty = std::max(0, std::min(smfReadMap->numBigTexY - 1, camBty));
@@ -703,9 +652,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 		int ydx = y * smfReadMap->heightMapSizeX;
 		int nloop = (xe - xs) / lod + 1;
 
-		//! EnlargeArrays(nVertices, nStrips [, stripSize])
-		//! includes one extra for final endstrip
-		ma->EnlargeArrays((52 * nloop), 14 * nloop + 1);
+		ma->EnlargeArrays(52 * nloop);
 
 		for (x = xs; x < xe; x += lod) {
 			int xlod = x + lod;
@@ -733,25 +680,25 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					const float h4 = (GetVisibleVertexHeight(ylhdx) + GetVisibleVertexHeight(ylhdx+lod)) * hmocxp + GetVisibleVertexHeight(ylhdx+hlod) * oldcamxpart;
 
 					if(inStrip){
-						EndStripQ(ma);
+						ma->EndStrip();
 						inStrip=false;
 					}
 					DrawVertexAQ(ma, x,y);
 					DrawVertexAQ(ma, x,yhlod,h1);
 					DrawVertexAQ(ma, xhlod,y,h2);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
-					EndStripQ(ma);
+					ma->EndStrip();
 					DrawVertexAQ(ma, x,yhlod,h1);
 					DrawVertexAQ(ma, x,ylod);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, xhlod,ylod,h4);
-					EndStripQ(ma);
+					ma->EndStrip();
 					DrawVertexAQ(ma, xhlod,ylod,h4);
 					DrawVertexAQ(ma, xlod,ylod);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, xlod,y);
 					DrawVertexAQ(ma, xhlod,y,h2);
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 				if (x <= cx - vrhlod) {
 					const float h1 = (GetVisibleVertexHeight(yhdx+lod) + GetVisibleVertexHeight(ylhdx+lod)) * hocxp + GetVisibleVertexHeight(yhhdx+lod ) * mocxp;
@@ -760,25 +707,25 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					const float h4 = (GetVisibleVertexHeight(ylhdx   ) + GetVisibleVertexHeight(ylhdx+lod)) * hocxp + GetVisibleVertexHeight(ylhdx+hlod) * mocxp;
 
 					if(inStrip){
-						EndStripQ(ma);
+						ma->EndStrip();
 						inStrip=false;
 					}
 					DrawVertexAQ(ma, xlod,yhlod,h1);
 					DrawVertexAQ(ma, xlod,y);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, xhlod,y,h2);
-					EndStripQ(ma);
+					ma->EndStrip();
 					DrawVertexAQ(ma, xlod,ylod);
 					DrawVertexAQ(ma, xlod,yhlod,h1);
 					DrawVertexAQ(ma, xhlod,ylod,h4);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
-					EndStripQ(ma);
+					ma->EndStrip();
 					DrawVertexAQ(ma, xhlod,y,h2);
 					DrawVertexAQ(ma, x,y);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, x,ylod);
 					DrawVertexAQ(ma, xhlod,ylod,h4);
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 				if (y >= cy + vrhlod) {
 					const float h1 = (GetVisibleVertexHeight(yhdx     ) + GetVisibleVertexHeight(yhdx+lod)) * hmocyp + GetVisibleVertexHeight(yhdx+hlod ) * oldcamypart;
@@ -787,7 +734,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					const float h4 = (GetVisibleVertexHeight(ylhdx+lod) + GetVisibleVertexHeight(yhdx+lod)) * hmocyp + GetVisibleVertexHeight(yhhdx+lod ) * oldcamypart;
 
 					if(inStrip){
-						EndStripQ(ma);
+						ma->EndStrip();
 						inStrip=false;
 					}
 					DrawVertexAQ(ma, x,y);
@@ -796,13 +743,13 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, xlod,y);
 					DrawVertexAQ(ma, xlod,yhlod,h4);
-					EndStripQ(ma);
+					ma->EndStrip();
 					DrawVertexAQ(ma, x,yhlod,h2);
 					DrawVertexAQ(ma, x,ylod);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, xlod,ylod);
 					DrawVertexAQ(ma, xlod,yhlod,h4);
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 				if (y <= cy - vrhlod) {
 					const float h1 = (GetVisibleVertexHeight(ylhdx    ) + GetVisibleVertexHeight(ylhdx+lod)) * hocyp + GetVisibleVertexHeight(ylhdx+hlod) * mocyp;
@@ -811,7 +758,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					const float h4 = (GetVisibleVertexHeight(ylhdx+lod) + GetVisibleVertexHeight(yhdx+lod )) * hocyp + GetVisibleVertexHeight(yhhdx+lod ) * mocyp;
 
 					if (inStrip) {
-						EndStripQ(ma);
+						ma->EndStrip();
 						inStrip = false;
 					}
 					DrawVertexAQ(ma, x,yhlod,h2);
@@ -820,18 +767,18 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					DrawVertexAQ(ma, xhlod,ylod,h1);
 					DrawVertexAQ(ma, xlod,yhlod,h4);
 					DrawVertexAQ(ma, xlod,ylod);
-					EndStripQ(ma);
+					ma->EndStrip();
 					DrawVertexAQ(ma, xlod,yhlod,h4);
 					DrawVertexAQ(ma, xlod,y);
 					DrawVertexAQ(ma, xhlod,yhlod,h3);
 					DrawVertexAQ(ma, x,y);
 					DrawVertexAQ(ma, x,yhlod,h2);
-					EndStripQ(ma);
+					ma->EndStrip();
 				}
 			}
 		}
 		if (inStrip) {
-			EndStripQ(ma);
+			ma->EndStrip();
 			inStrip=false;
 		}
 	}
@@ -839,9 +786,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 	int yst = std::max(ystart - lod, minty);
 	int yed = std::min(yend + lod, maxty);
 	int nloop = (yed - yst) / lod + 1;
-
-	if (nloop > 0)
-		ma->EnlargeArrays((8 * nloop), 2 * nloop);
+	ma->EnlargeArrays(8 * nloop);
 
 	//!rita yttre begr?snings yta mot n?ta lod
 	if (maxlx < maxtx && maxlx >= mintx) {
@@ -862,7 +807,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 				DrawVertexAQ(ma, xlod,y);
 				DrawVertexAQ(ma, xlod,y+lod,h);
 			}
-			EndStripQ(ma);
+			ma->EndStrip();
 		}
 	}
 
@@ -883,7 +828,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 			}
 			DrawVertexAQ(ma, xlod,y);
 			DrawVertexAQ(ma, xlod,y+lod);
-			EndStripQ(ma);
+			ma->EndStrip();
 		}
 	}
 	if (maxly < maxty && maxly > minty) {
@@ -897,7 +842,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 			const int ydx = y * smfReadMap->heightMapSizeX;
 			const int nloop = (xe - xs) / lod + 2; //! two extra for if statment
 
-			ma->EnlargeArrays((2 * nloop), 1);
+			ma->EnlargeArrays(2 * nloop);
 
 			if (x % dlod) {
 				const int ylhdx = ydx + x + lhdx;
@@ -920,7 +865,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					DrawVertexAQ(ma, x+lod,ylod,h);
 				}
 			}
-			EndStripQ(ma);
+			ma->EndStrip();
 		}
 	}
 	if (minly > minty && minly < maxty) {
@@ -934,7 +879,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 			const int ydx = y * smfReadMap->heightMapSizeX;
 			const int nloop = (xe - xs) / lod + 2; //! two extra for if statment
 
-			ma->EnlargeArrays((2 * nloop), 1);
+			ma->EnlargeArrays(2 * nloop);
 
 			if (x % dlod) {
 				const int yhdx = ydx + x;
@@ -957,7 +902,7 @@ void CLegacyMeshDrawer::DoDrawGroundShadowLOD(int nlod) {
 					DrawVertexAQ(ma, x + lod, ylod);
 				}
 			}
-			EndStripQ(ma);
+			ma->EndStrip();
 		}
 	}
 	DrawGroundVertexArrayQ(ma);
@@ -968,26 +913,6 @@ void CLegacyMeshDrawer::DrawShadowMesh()
 {
 	{ // profiler scope
 		const int NUM_LODS = 4;
-#ifdef USE_GML
-		// Profiler results, 4 threads: multiThreadDrawGroundShadow is rarely faster than single threaded rendering (therefore disabled by default)
-		const bool mt = GML_PROFILER(multiThreadDrawGroundShadow)
-
-		if (mt) {
-			gmlProcessor->Work(
-				NULL,                                       // wrk
-				&CLegacyMeshDrawer::DoDrawGroundShadowLODMT, // wrka
-				NULL,                                       // wrkit
-				this,                                       // cls
-				gmlThreadCount,                             // mt
-				FALSE,                                      // sm
-				NULL,                                       // it
-				NUM_LODS + 1,                               // nu
-				50,                                         // l1
-				100,                                        // l2
-				TRUE                                        // sw
-			);
-		} else
-#endif
 		{
 			for (int nlod = 0; nlod < NUM_LODS + 1; ++nlod) {
 				DoDrawGroundShadowLOD(nlod);
