@@ -203,7 +203,7 @@ void CAdvWater::Draw(bool useBlending)
 
 				xbase += dh;
 			}
-			va->EndStripQ();
+			va->EndStrip();
 			base += dv;
 			screenY -= yInc;
 		}
@@ -276,6 +276,7 @@ void CAdvWater::UpdateWater(CGame* game)
 
 	va->DrawArrayT(GL_QUADS);
 
+	va = GetVertexArray();
 	va->Initialize();
 	glBindTexture(GL_TEXTURE_2D, rawBumpTexture[1]);
 
@@ -286,6 +287,7 @@ void CAdvWater::UpdateWater(CGame* game)
 
 	va->DrawArrayT(GL_QUADS);
 
+	va = GetVertexArray();
 	va->Initialize();
 	glBindTexture(GL_TEXTURE_2D, rawBumpTexture[2]);
 
