@@ -432,8 +432,8 @@ void CDecalsDrawerGL4::CreateBoundingBoxVBOs()
 	vboVertices.Bind(GL_ARRAY_BUFFER);
 	vboIndices.Bind(GL_ELEMENT_ARRAY_BUFFER);
 
-	vboVertices.Resize(sizeof(boxverts) * sizeof(float3), GL_STATIC_DRAW, &boxverts[0]);
-	vboIndices.Resize(sizeof(indices) * sizeof(GLubyte), GL_STATIC_DRAW, &indices[0]);
+	vboVertices.New(sizeof(boxverts) * sizeof(float3), GL_STATIC_DRAW, &boxverts[0]);
+	vboIndices.New(sizeof(indices) * sizeof(GLubyte), GL_STATIC_DRAW, &indices[0]);
 
 	vboVertices.Unbind();
 	vboIndices.Unbind();
@@ -444,6 +444,7 @@ void CDecalsDrawerGL4::CreateStructureVBOs()
 {
 	{
 	GLuint uniformBlockIndex = glGetUniformBlockIndex(decalShader->GetObjID(), "SGroundLighting");
+	assert(uniformBlockIndex != GL_INVALID_INDEX);
 
 	GLsizei uniformBlockSize = 0;
 	glGetActiveUniformBlockiv(decalShader->GetObjID(), uniformBlockIndex, GL_UNIFORM_BLOCK_DATA_SIZE, &uniformBlockSize);
@@ -458,7 +459,7 @@ void CDecalsDrawerGL4::CreateStructureVBOs()
 
 
 	uboGroundLighting.Bind(GL_UNIFORM_BUFFER);
-	uboGroundLighting.Resize(uniformBlockSize, GL_STATIC_DRAW);
+	uboGroundLighting.New(uniformBlockSize, GL_STATIC_DRAW);
 		SGLSLGroundLighting* uboGroundLightingData = (SGLSLGroundLighting*)uboGroundLighting.MapBuffer(0, sizeof(SGLSLGroundLighting));
 		uboGroundLightingData->ambientColor  = mapInfo->light.groundAmbientColor  * CGlobalRendering::SMF_INTENSITY_MULT;
 		uboGroundLightingData->diffuseColor  = mapInfo->light.groundSunColor      * CGlobalRendering::SMF_INTENSITY_MULT;
@@ -476,7 +477,8 @@ void CDecalsDrawerGL4::CreateStructureVBOs()
 	{
 	uboDecalsStructures.Bind(GL_UNIFORM_BUFFER);
 
-	GLuint uniformBlockIndex = glGetUniformBlockIndex(decalShader->GetObjID(), "decals");
+	// Uniform Array Solution
+	/*GLuint uniformBlockIndex = glGetUniformBlockIndex(decalShader->GetObjID(), "decals");
 
 	GLsizei uniformBlockSize = 0;
 	glGetActiveUniformBlockiv(decalShader->GetObjID(), uniformBlockIndex, GL_UNIFORM_BLOCK_DATA_SIZE, &uniformBlockSize);
@@ -487,16 +489,20 @@ void CDecalsDrawerGL4::CreateStructureVBOs()
 
 	maxDecals = uniformBlockSize / sizeof(SGLSLDecal);
 
+	uniformBlockSize = maxDecals * sizeof(SGLSLDecal);
+	uboDecalsStructures.New(uniformBlockSize, GL_DYNAMIC_DRAW);
+
+	glUniformBlockBinding(decalShader->GetObjID(), uniformBlockIndex, 3);
+	glBindBufferBase(GL_UNIFORM_BUFFER, 3, uboDecalsStructures.GetId());*/
+
+	// TBO solution
 	GLint maxTexBufSize;
 	glGetIntegerv(GL_MAX_TEXTURE_BUFFER_SIZE, &maxTexBufSize);
 
 	maxDecals = (maxTexBufSize / sizeof(SGLSLDecal)) / 2;
 
-	uniformBlockSize = maxDecals * sizeof(SGLSLDecal);
-	uboDecalsStructures.Resize(uniformBlockSize, GL_DYNAMIC_DRAW);
-
-	glUniformBlockBinding(decalShader->GetObjID(), uniformBlockIndex, 3);
-	glBindBufferBase(GL_UNIFORM_BUFFER, 3, uboDecalsStructures.GetId());
+	GLsizei uniformBlockSize = maxDecals * sizeof(SGLSLDecal);
+	uboDecalsStructures.New(uniformBlockSize, GL_DYNAMIC_DRAW);
 
 	uboDecalsStructures.Unbind();
 	}
@@ -506,7 +512,7 @@ void CDecalsDrawerGL4::CreateStructureVBOs()
 
 	/*{
 	vboVisibilityFeeback.Bind(GL_UNIFORM_BUFFER);
-	vboVisibilityFeeback.Resize(maxDecals, GL_STATIC_DRAW);
+	vboVisibilityFeeback.New(maxDecals, GL_STATIC_DRAW);
 	vboVisibilityFeeback.Unbind();
 	}*/
 }
@@ -708,7 +714,7 @@ void CDecalsDrawerGL4::AddExplosion(float3 pos, float damage, float radius, bool
 	//	return;
 
 	//FIXME decalLevel is private! const float lifeTime = decalLevel * damage * 3.0f;
-	const float altitude = pos.y - ground->GetHeightReal(pos.x, pos.z, false);
+	const float altitude = pos.y - CGround::GetHeightReal(pos.x, pos.z, false);
 
 	// no decals for below-ground & in-air explosions
 	if (abs(altitude) > radius) { return; }
@@ -737,8 +743,6 @@ void CDecalsDrawerGL4::AddExplosion(float3 pos, float damage, float radius, bool
 	SNPRINTF(buf, sizeof(buf), "%i", r);
 	s->texOffsets = atlasTexs[std::string(buf)];
 
-	GML_STDMUTEX_LOCK(decal);
-
 	if (decals.size() < maxDecals) {
 		//FIXME use mt-safe container
 		decals.push_back(s);
@@ -756,8 +760,6 @@ void CDecalsDrawerGL4::UnitCreated(const CUnit* unit, const CUnit* builder)
 	if (!unit->unitDef->decalDef.useGroundDecal)
 		return;
 
-	GML_STDMUTEX_LOCK(decal);
-
 	const int sizex = unit->unitDef->decalDef.groundDecalSizeX;
 	const int sizey = unit->unitDef->decalDef.groundDecalSizeY;
 
@@ -785,8 +787,6 @@ void CDecalsDrawerGL4::UnitDestroyed(const CUnit* unit, const CUnit* attacker)
 	if (!unit->unitDef->decalDef.useGroundDecal)
 		return;
 
-	GML_STDMUTEX_LOCK(decal);
-
 	//TODO FINISH
 	//decal->owner = NULL;
 	//decal->gbOwner = gb;
