@@ -221,7 +221,7 @@ inline void CGroundDecalHandler::DrawObjectDecal(SolidObjectGroundDecal* decal)
 	const int gsmx1 = gsmx + 1;
 	const int gsmy = gs->mapy;
 
-	unsigned char color[4] = {255, 255, 255, (unsigned char)(decal->alpha * 255)};
+	SColor color(255, 255, 255, int(decal->alpha * 255));
 
 	#ifndef DEBUG
 	#define HEIGHT(z, x) (hm[((z) * gsmx1) + (x)])
@@ -297,19 +297,17 @@ inline void CGroundDecalHandler::DrawObjectDecal(SolidObjectGroundDecal* decal)
 			}
 		}
 	} else {
-		const float c = *((float*) (color));
-		const int start = 0;
-		const int stride = 6;
-		const int sdi = decal->va->drawIndex();
+		const int num = decal->va->drawIndex() / VA_SIZE_TC;
+		decal->va->ResetPos();
+		VA_TYPE_TC* mem = decal->va->GetTypedVertexArray<VA_TYPE_TC>(num);
 
-		for (int i = start; i < sdi; i += stride) {
-			const int x = int(decal->va->drawArray[i + 0]) >> 3;
-			const int z = int(decal->va->drawArray[i + 2]) >> 3;
-			const float h = hm[z * gsmx1 + x];
+		for (int i = 0; i < num; ++i) {
+			const int x = int(mem[i].p.x) >> 3;
+			const int z = int(mem[i].p.z) >> 3;
 
 			// update the height and alpha
-			decal->va->drawArray[i + 1] = h;
-			decal->va->drawArray[i + 5] = c;
+			mem[i].p.y = hm[z * gsmx1 + x];
+			mem[i].c   = color;
 		}
 
 		decal->va->DrawArrayTC(GL_QUADS);
@@ -325,12 +323,7 @@ inline void CGroundDecalHandler::DrawGroundScar(CGroundDecalHandler::Scar* scar,
 	if (!camera->InView(scar->pos, scar->radius + 16))
 		return;
 
-	const float* hm = readMap->GetCornerHeightMapUnsynced();
-
-	const int gsmx = gs->mapx;
-	const int gsmx1 = gsmx + 1;
-
-	unsigned char color[4] = {255, 255, 255, 255};
+	SColor color(255, 255, 255, 255);
 
 	if (scar->va == NULL) {
 		scar->va = new CVertexArray();
@@ -350,7 +343,6 @@ inline void CGroundDecalHandler::DrawGroundScar(CGroundDecalHandler::Scar* scar,
 		// create the scar texture-quads
 		float px1 = sx * 16;
 		for (int x = sx; x <= ex; ++x) {
-			//const float* hm2 = hm;
 			float px2 = px1 + 16;
 			float pz1 = sz * 16;
 
@@ -360,20 +352,18 @@ inline void CGroundDecalHandler::DrawGroundScar(CGroundDecalHandler::Scar* scar,
 				float tx2 = max(0.0f, (pos.x - px2) / radius4 + 0.25f);
 				float tz1 = min(0.5f, (pos.z - pz1) / radius4 + 0.25f);
 				float tz2 = max(0.0f, (pos.z - pz2) / radius4 + 0.25f);
-				float h1 = ground->GetHeightReal(px1, pz1, false);
-				float h2 = ground->GetHeightReal(px2, pz1, false);
-				float h3 = ground->GetHeightReal(px2, pz2, false);
-				float h4 = ground->GetHeightReal(px1, pz2, false);
+				float h1 = CGround::GetHeightReal(px1, pz1, false);
+				float h2 = CGround::GetHeightReal(px2, pz1, false);
+				float h3 = CGround::GetHeightReal(px2, pz2, false);
+				float h4 = CGround::GetHeightReal(px1, pz2, false);
 
 				scar->va->AddVertexTC(float3(px1, h1, pz1), tx1 + tx, tz1 + ty, color);
 				scar->va->AddVertexTC(float3(px2, h2, pz1), tx2 + tx, tz1 + ty, color);
 				scar->va->AddVertexTC(float3(px2, h3, pz2), tx2 + tx, tz2 + ty, color);
 				scar->va->AddVertexTC(float3(px1, h4, pz2), tx1 + tx, tz2 + ty, color);
-				//hm2 += gsmx12;
 				pz1 = pz2;
 			}
 
-			//hm2 += 2;
 			px1 = px2;
 		}
 	} else {
@@ -384,17 +374,20 @@ inline void CGroundDecalHandler::DrawGroundScar(CGroundDecalHandler::Scar* scar,
 				color[3] = (int) (scar->startAlpha - (gs->frameNum - scar->creationTime) * scar->alphaFalloff);
 			}
 
-			const int start = 0;
-			const int stride = 6;
-			const int sdi = scar->va->drawIndex();
+			const int gsmx1 = gs->mapx + 1;
+			const float* hm = readMap->GetCornerHeightMapUnsynced();
 
-			for (int i = start; i < sdi; i += stride) {
-				const int x = int(scar->va->drawArray[i + 0]) >> 3;
-				const int z = int(scar->va->drawArray[i + 2]) >> 3;
+			const int num = scar->va->drawIndex() / VA_SIZE_TC;
+			scar->va->ResetPos();
+			VA_TYPE_TC* mem = scar->va->GetTypedVertexArray<VA_TYPE_TC>(num);
+
+			for (int i = 0; i < num; ++i) {
+				const int x = int(mem[i].p.x) >> 3;
+				const int z = int(mem[i].p.z) >> 3;
 
 				// update the height and alpha
-				scar->va->drawArray[i + 1] = hm[z * gsmx1 + x];
-				scar->va->drawArray[i + 5] = *reinterpret_cast<float*>(color);
+				mem[i].p.y = hm[z * gsmx1 + x];
+				mem[i].c   = color;
 			}
 		}
 
@@ -491,7 +484,6 @@ void CGroundDecalHandler::DrawObjectDecals() {
 		glBindTexture(GL_TEXTURE_2D, decalType->texture);
 
 		{
-			GML_STDMUTEX_LOCK(decal); // Draw
 			GatherDecalsForType(decalType);
 		}
 
@@ -507,8 +499,6 @@ void CGroundDecalHandler::DrawObjectDecals() {
 
 void CGroundDecalHandler::AddTracks() {
 	{
-		GML_STDMUTEX_LOCK(track); // AddTracks
-
 		// Delayed addition of new tracks
 		for (std::vector<TrackToAdd>::iterator ti = tracksToBeAdded.begin(); ti != tracksToBeAdded.end(); ++ti) {
 			const TrackToAdd* tta = &(*ti);
@@ -626,9 +616,8 @@ void CGroundDecalHandler::DrawTracks() {
 	}
 }
 
-void CGroundDecalHandler::CleanTracks() {
-	GML_STDMUTEX_LOCK(track); // CleanTracks
-
+void CGroundDecalHandler::CleanTracks()
+{
 	// Cleanup old tracks
 	for (std::vector<TrackToClean>::iterator ti = tracksToBeCleaned.begin(); ti != tracksToBeCleaned.end(); ++ti) {
 		TrackToClean* ttc = &(*ti);
@@ -656,12 +645,11 @@ void CGroundDecalHandler::CleanTracks() {
 
 
 
-void CGroundDecalHandler::AddScars() {
+void CGroundDecalHandler::AddScars()
+{
 	scarsToBeChecked.clear();
 
 	{
-		GML_STDMUTEX_LOCK(scar); // AddScars
-
 		for (std::vector<Scar*>::iterator si = scarsToBeAdded.begin(); si != scarsToBeAdded.end(); ++si)
 			scarsToBeChecked.push_back(*si);
 
@@ -891,16 +879,14 @@ void CGroundDecalHandler::AddDecalAndTrack(CUnit* unit, const float3& newPos)
 	TrackPart* tp = new TrackPart();
 	tp->pos1 = pos + unit->rightdir * decalDef.trackDecalWidth * 0.5f;
 	tp->pos2 = pos - unit->rightdir * decalDef.trackDecalWidth * 0.5f;
-	tp->pos1.y = ground->GetHeightReal(tp->pos1.x, tp->pos1.z, false);
-	tp->pos2.y = ground->GetHeightReal(tp->pos2.x, tp->pos2.z, false);
+	tp->pos1.y = CGround::GetHeightReal(tp->pos1.x, tp->pos1.z, false);
+	tp->pos2.y = CGround::GetHeightReal(tp->pos2.x, tp->pos2.z, false);
 	tp->creationTime = gs->frameNum;
 
 	TrackToAdd tta;
 	tta.tp = tp;
 	tta.unit = unit;
 
-	GML_STDMUTEX_LOCK(track); // AddDecalAndTrack
-
 	if (unit->myTrack == NULL) {
 		unit->myTrack = new UnitTrackStruct(unit);
 		unit->myTrack->lifeTime = trackLifeTime;
@@ -946,8 +932,6 @@ int CGroundDecalHandler::GetTrackType(const std::string& name)
 	tt->name = lowerName;
 	tt->texture = LoadTexture(lowerName);
 
-//	GML_STDMUTEX_LOCK(tracktype); // GetTrackType
-
 	trackTypes.push_back(tt);
 
 	return (trackTypes.size() - 1);
@@ -993,7 +977,7 @@ void CGroundDecalHandler::AddExplosion(float3 pos, float damage, float radius, b
 	if (decalLevel == 0 || !addScar)
 		return;
 
-	const float altitude = pos.y - ground->GetHeightReal(pos.x, pos.z, false);
+	const float altitude = pos.y - CGround::GetHeightReal(pos.x, pos.z, false);
 
 	// no decals for below-ground explosions
 	if (altitude <= -1.0f)
@@ -1036,8 +1020,6 @@ void CGroundDecalHandler::AddExplosion(float3 pos, float damage, float radius, b
 	s->overdrawn = 0;
 	s->lastTest = 0;
 
-	GML_STDMUTEX_LOCK(scar); // AddExplosion
-
 	scarsToBeAdded.push_back(s);
 }
 
@@ -1175,8 +1157,6 @@ void CGroundDecalHandler::MoveSolidObject(CSolidObject* object, const float3& po
 			return;
 	}
 
-	GML_STDMUTEX_LOCK(decal); // AddSolidObject
-
 	SolidObjectGroundDecal* olddecal = object->groundDecal;
 	if (olddecal != NULL) {
 		olddecal->owner = NULL;
@@ -1218,8 +1198,6 @@ void CGroundDecalHandler::RemoveSolidObject(CSolidObject* object, GhostSolidObje
 	if (decalLevel == 0)
 		return;
 
-	GML_STDMUTEX_LOCK(decal); // RemoveSolidObject
-
 	assert(object);
 	SolidObjectGroundDecal* decal = object->groundDecal;
 
@@ -1243,8 +1221,6 @@ void CGroundDecalHandler::ForceRemoveSolidObject(CSolidObject* object)
 	if (decalLevel == 0)
 		return;
 
-	GML_STDMUTEX_LOCK(decal); // ForcedRemoveSolidObject
-
 	SolidObjectGroundDecal* decal = object->groundDecal;
 
 	if (decal == NULL)
@@ -1266,8 +1242,6 @@ int CGroundDecalHandler::GetSolidObjectDecalType(const std::string& name)
 
 	int decalType = 0;
 
-	GML_STDMUTEX_LOCK(decal); // GetSolidObjectDecalType
-
 	std::vector<SolidObjectDecalType*>::iterator bi;
 	for (bi = objectDecalTypes.begin(); bi != objectDecalTypes.end(); ++bi) {
 		if ((*bi)->name == lowerName) {
@@ -1322,8 +1296,6 @@ void CGroundDecalHandler::RenderUnitDestroyed(const CUnit* unit) {
 	if (decalLevel == 0)
 		return;
 
-	GML_STDMUTEX_LOCK(track); // RemoveUnit
-
 	CUnit* u = const_cast<CUnit*>(unit);
 	RemoveSolidObject(u, NULL);
 
